# -*- coding: utf-8 -*-
# Copyright (C) 2015-2018 Linaro Limited
#
# Author: Stevan Radakovic <stevan.radakovic@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=no-member,too-many-locals,too-many-nested-blocks,
# pylint: disable=too-many-return-statements,ungrouped-imports

import hashlib
import os
import yaml
import logging
import decimal
from urllib.parse import quote

from collections import OrderedDict  # pylint: disable=unused-import

from lava_common.utils import debian_package_version
from lava_results_app.models import (
    TestSuite,
    TestSet,
    TestCase,
    TestData,
    ActionData,
    MetaType,
)
from django.core.exceptions import MultipleObjectsReturned
from lava_common.timeout import Timeout


def yaml_decimal_str(dumper, value):
    return yaml.ScalarNode(tag=u"tag:yaml.org,2002:str", value=str(value))


yaml.add_representer(decimal.Decimal, yaml_decimal_str)


def _check_for_testset(result_dict, suite):
    """
    The presence of the test_set key indicates the start and usage of a TestSet.
    Get or create and populate the definition based on that set.
    # {date: pass, test_definition: install-ssh, test_set: first_set}
    :param result_dict: lava-test-shell results
    :param suite: current test suite
    """
    logger = logging.getLogger("lava-master")
    testset = None
    if "set" in result_dict:
        set_name = result_dict["set"]
        if set_name != quote(set_name):
            msg = "Invalid testset name '%s', ignoring." % set_name
            suite.job.set_failure_comment(msg)
            logger.warning(msg)
            return None
        testset, _ = TestSet.objects.get_or_create(name=set_name, suite=suite)
        logger.debug("%s", testset)
    return testset


def append_failure_comment(job, msg):
    if not job.failure_comment:
        job.failure_comment = ""
    job.failure_comment += msg[:256]
    job.save(update_fields=["failure_comment"])


def create_metadata_store(results, job):
    """
    Uses the OrderedDict import to correctly handle
    the yaml.load
    """
    if "extra" not in results:
        return None
    level = results.get("level")
    if level is None:
        return None

    logger = logging.getLogger("lava-master")
    stub = "%s-%s-%s.yaml" % (results["definition"], results["case"], level)
    meta_filename = os.path.join(job.output_dir, "metadata", stub)
    os.makedirs(os.path.dirname(meta_filename), mode=0o755, exist_ok=True)
    if os.path.exists(meta_filename):
        with open(meta_filename, "r") as existing_store:
            data = yaml.load(existing_store)
        data.update(results["extra"])
    else:
        data = results["extra"]
    try:
        with open(meta_filename, "w") as extra_store:
            yaml.dump(data, extra_store)
    except OSError as exc:  # LAVA-847
        msg = "[%d] Unable to create metadata store: %s" % (job.id, exc)
        logger.error(msg)
        append_failure_comment(job, msg)
        return None
    return meta_filename


def map_scanned_results(
    results, job, markers, meta_filename
):  # pylint: disable=too-many-branches,too-many-statements,too-many-return-statements
    """
    Sanity checker on the logged results dictionary
    :param results: results logged via the slave
    :param job: the current test job
    :param meta_filename: YAML store for results metadata
    :return: the TestCase object that should be saved to the database.
             None on error.
    """
    logger = logging.getLogger("lava-master")

    if not isinstance(results, dict):
        append_failure_comment(job, "[%d] %s is not a dictionary" % (job.id, results))
        return None

    if not {"definition", "case", "result"}.issubset(set(results.keys())):
        append_failure_comment(
            job, 'Missing some keys ("definition", "case" or "result") in %s' % results
        )
        return None

    if "extra" in results:
        results["extra"] = meta_filename

    metadata = yaml.dump(results)
    if len(metadata) > 4096:  # bug 2471 - test_length unit test
        msg = "[%d] Result metadata is too long. %s" % (job.id, metadata)
        logger.error(msg)
        append_failure_comment(job, msg)
        metadata = ""

    suite, _ = TestSuite.objects.get_or_create(name=results["definition"], job=job)
    testset = _check_for_testset(results, suite)

    name = results["case"].strip()

    test_case = None
    if suite.name == "lava":
        try:
            result_val = TestCase.RESULT_MAP[results["result"]]
        except KeyError:
            logger.error('[%d] Unable to MAP result "%s"', job.id, results["result"])
            return None

        measurement = None
        units = ""
        if "duration" in results:
            measurement = results["duration"]
            units = "seconds"
        test_case = TestCase(
            name=name,
            suite=suite,
            test_set=testset,
            metadata=metadata,
            measurement=measurement,
            units=units,
            result=result_val,
        )

    else:
        result = results["result"]
        measurement = None
        units = ""
        if testset:
            logger.debug("%s/%s/%s %s", suite, testset, name, result)
        else:
            logger.debug("%s/%s %s", suite, name, result)
        if "measurement" in results:
            measurement = results["measurement"]
        if "units" in results:
            units = results["units"]
            logger.debug("%s/%s %s%s", suite, name, measurement, units)
        if result not in TestCase.RESULT_MAP:
            logger.warning(
                "[%d] Unrecognised result: '%s' for test case '%s'",
                job.id,
                result,
                name,
            )
            return None
        # Add the marker if available
        starttc = endtc = None
        if name in markers:
            test_case = markers[name].get("test_case")
            starttc = markers[name].get("start_test_case", test_case)
            endtc = markers[name].get("end_test_case", test_case)
            del markers[name]
        try:
            test_case = TestCase(
                name=name,
                suite=suite,
                test_set=testset,
                result=TestCase.RESULT_MAP[result],
                metadata=metadata,
                measurement=measurement,
                units=units,
                start_log_line=starttc,
                end_log_line=endtc,
            )
        except decimal.InvalidOperation:
            logger.exception("[%d] Unable to create test case %s", job.id, name)
    return test_case


def _add_parameter_metadata(prefix, definition, dictionary, label):
    if "parameters" in definition and isinstance(definition["parameters"], dict):
        for paramkey, paramvalue in definition["parameters"].items():
            if paramkey == "yaml_line":
                continue
            dictionary["%s.%s.parameters.%s" % (prefix, label, paramkey)] = paramvalue


def _get_job_metadata(job):
    retval = {}
    # Add original_definition checksum to metadata
    retval.update(
        {
            "definition-checksum": hashlib.md5(  # nosec - not used for crypto
                job.original_definition.encode("utf-8")
            ).hexdigest()
        }
    )
    # Add lava-server-version to metadata
    packaged = debian_package_version(pkg="lava-server", split=False)
    if packaged:
        retval.update({"lava-server-version": packaged})
    return retval


def _get_action_metadata(
    data
):  # pylint: disable=too-many-branches,too-many-nested-blocks,too-many-statements
    if not isinstance(data, list):
        return None
    retval = {}
    for action in data:
        deploy = [dict.get(action, "deploy")]
        count = 0
        for block in deploy:
            if not block:
                continue
            namespace = block.get("namespace")
            prefix = (
                "deploy.%d.%s" % (count, namespace)
                if namespace
                else "deploy.%d" % count
            )
            value = block.get("method")
            if value:
                retval["%s.method" % prefix] = value
                count += 1
        boot = [dict.get(action, "boot")]
        count = 0
        for block in boot:
            if not block:
                continue
            namespace = block.get("namespace")
            prefix = (
                "boot.%d.%s" % (count, namespace) if namespace else "boot.%d" % count
            )
            value = block.get("commands")
            if value:
                retval["%s.commands" % prefix] = value
            value = block.get("method")
            if value:
                retval["%s.method" % prefix] = value
            value = block.get("type")
            if value:
                retval["%s.type" % prefix] = value
            count += 1
        test = [dict.get(action, "test")]
        count = 0
        for block in test:
            if not block:
                continue
            namespace = block.get("namespace")
            definitions = [dict.get(block, "definitions")][0]
            if not definitions:
                monitors = [dict.get(block, "monitors")][0]
                if monitors:
                    if isinstance(monitors, list):
                        for monitor in monitors:
                            prefix = (
                                "test.%d.%s" % (count, namespace)
                                if namespace
                                else "test.%d" % count
                            )
                            retval["%s.monitor.name" % prefix] = monitor["name"]
                            count += 1
            else:
                for definition in definitions:
                    if definition["from"] == "inline":
                        run = definition["repository"].get("run")
                        # an inline repo without test cases will not get reported.
                        steps = [dict.get(run, "steps")][0] if run else None
                        if steps is not None and "lava-test-case" in steps:
                            prefix = (
                                "test.%d.%s" % (count, namespace)
                                if namespace
                                else "test.%d" % count
                            )
                        else:
                            # store the fact that an inline exists but would not generate any testcases
                            prefix = (
                                "omitted.%d.%s" % (count, namespace)
                                if namespace
                                else "omitted.%d" % count
                            )
                        retval["%s.inline.name" % prefix] = definition["name"]
                        _add_parameter_metadata(
                            prefix=prefix,
                            definition=definition,
                            dictionary=retval,
                            label="inline",
                        )
                        retval["%s.inline.path" % prefix] = definition["path"]
                    else:
                        prefix = (
                            "test.%d.%s" % (count, namespace)
                            if namespace
                            else "test.%d" % count
                        )
                        # FIXME: what happens with remote definition without lava-test-case?
                        retval["%s.definition.name" % prefix] = definition["name"]
                        retval["%s.definition.path" % prefix] = definition["path"]
                        retval["%s.definition.from" % prefix] = definition["from"]
                        retval["%s.definition.repository" % prefix] = definition[
                            "repository"
                        ]
                        _add_parameter_metadata(
                            prefix=prefix,
                            definition=definition,
                            dictionary=retval,
                            label="definition",
                        )
                    count += 1
    return retval


def build_action(action_data, testdata, submission):
    # test for a known section
    logger = logging.getLogger("lava-master")
    if "section" not in action_data:
        logger.warning("Invalid action data - missing section")
        return

    metatype = MetaType.get_section(action_data["section"])
    if metatype is None:  # 0 is allowed
        logger.debug("Unrecognised metatype in action_data: %s", action_data["section"])
        return
    # lookup the type from the job definition.
    type_name = MetaType.get_type_name(action_data, submission)
    if not type_name:
        logger.debug(
            "type_name failed for %s metatype %s",
            action_data["section"],
            MetaType.TYPE_CHOICES[metatype],
        )
        return
    action_meta, _ = MetaType.objects.get_or_create(name=type_name, metatype=metatype)
    max_retry = action_data.get("max_retries")

    # find corresponding test case
    match_case = None
    test_cases = TestCase.objects.filter(
        suite__job=testdata.testjob, suite__name="lava"
    )
    for case in test_cases:
        if case.action_metadata:
            if case.action_metadata.get("level") == action_data["level"]:
                match_case = case

    # maps the static testdata derived from the definition to the runtime pipeline construction
    ActionData.objects.create(
        action_name=action_data["name"],
        action_level=action_data["level"],
        action_summary=action_data["summary"],
        testdata=testdata,
        action_description=action_data["description"],
        meta_type=action_meta,
        max_retries=max_retry,
        timeout=int(Timeout.parse(action_data["timeout"])),
        testcase=match_case,
    )


def walk_actions(data, testdata, submission):
    for action in data:
        build_action(action, testdata, submission)
        if "pipeline" in action:
            walk_actions(action["pipeline"], testdata, submission)


def map_metadata(description, job):
    """
    Generate metadata from the combination of the pipeline definition
    file (after any parsing for protocols) and the pipeline description
    into static metadata (TestData) related to this specific job
    The description itself remains outside the database - it will need
    to be made available as a download link.
    :param description: the pipeline description output
    :param job: the TestJob to associate
    :return: True on success, False on error
    """
    logger = logging.getLogger("lava-master")
    try:
        submission_data = yaml.safe_load(job.definition)
        description_data = yaml.load(description)
    except yaml.YAMLError as exc:
        logger.exception("[%s] %s", job.id, exc)
        return False
    try:
        testdata, created = TestData.objects.get_or_create(testjob=job)
    except MultipleObjectsReturned:
        # only happens for small number of jobs affected by original bug.
        logger.info("[%s] skipping alteration of duplicated TestData", job.id)
        return False
    if not created:
        # prevent updates of existing TestData
        logger.debug("[%s] skipping alteration of existing TestData", job.id)
        return False

    # get job-action metadata
    if description is None:
        logger.warning("[%s] skipping empty description", job.id)
        return False
    if not description_data:
        logger.warning("[%s] skipping invalid description data", job.id)
        return False
    if "job" not in description_data:
        logger.warning("[%s] skipping description without a job.", job.id)
        return False
    action_values = _get_action_metadata(description_data["job"]["actions"])
    for key, value in action_values.items():
        if not key or not value:
            logger.warning("[%s] Missing element in job. %s: %s", job.id, key, value)
            continue
        testdata.attributes.create(name=key, value=value)

    # get common job metadata
    job_metadata = _get_job_metadata(job)
    for key, value in job_metadata.items():
        testdata.attributes.create(name=key, value=value)

    # get metadata from device
    device_values = {}
    device_values["target.device_type"] = job.requested_device_type
    for key, value in device_values.items():
        if not key or not value:
            logger.warning("[%s] Missing element in device. %s: %s", job.id, key, value)
            continue
        testdata.attributes.create(name=key, value=value)

    # Add metadata from job submission data.
    if "metadata" in submission_data:
        for key in submission_data["metadata"]:
            value = submission_data["metadata"][key]
            if not key or not value:
                logger.warning(
                    "[%s] Missing element in job. %s: %s", job.id, key, value
                )
                continue
            testdata.attributes.create(name=key, value=value)

    walk_actions(description_data["pipeline"], testdata, submission_data)
    return True


def testsuite_export_fields():
    """
    Keep this list in sync with the keys in export_testsuite
    :return: list of fields used in export_testsuite
    """
    return ["name", "job", "id"]


def export_testsuite(testsuite):
    """
    Returns string versions of selected elements of a TestSuite
    :param testsuite: TestSuite object
    :return: Dictionary containing relevant information formatted for export
    """
    suitedict = {
        "name": str(testsuite.name),
        "job": str(testsuite.job_id),
        "id": str(testsuite.id),
    }
    return suitedict



# Copyright (C) 2010, 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of linaro-django-xmlrpc.
#
# linaro-django-xmlrpc is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# linaro-django-xmlrpc is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-django-xmlrpc.  If not, see <http://www.gnu.org/licenses/>.


from django.conf.urls import url

from linaro_django_xmlrpc.views import (
    create_token,
    delete_token,
    delete_unused_tokens,
    edit_token,
    tokens,
)


urlpatterns = [
    url(r"^tokens/$", tokens, name="linaro_django_xmlrpc_tokens"),
    url(
        r"^tokens/create/$",
        create_token,
        name="linaro_django_xmlrpc.views.create_token",
    ),
    url(
        r"^tokens/(?P<object_id>\d+)/delete/$",
        delete_token,
        name="linaro_django_xmlrpc.views.delete_token",
    ),
    url(
        r"^tokens/delete_unused/$",
        delete_unused_tokens,
        name="linaro_django_xmlrpc.views.delete_unused_tokens",
    ),
    url(
        r"^tokens/(?P<object_id>\d+)/edit/$",
        edit_token,
        name="linaro_django_xmlrpc.views.edit_token",
    ),
]



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2018 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

import os
import contextlib
import csv

from django.core.management.base import BaseCommand, CommandError, CommandParser
from django.db import transaction
from django.contrib.auth.models import User, Group
from lava_scheduler_app.models import Device, DeviceType, Tag, Worker

# pylint: disable=bad-continuation


class Command(BaseCommand):
    help = "Manage devices"

    device_state = {
        "IDLE": Device.STATE_IDLE,
        "RESERVED": Device.STATE_RESERVED,
        "RUNNING": Device.STATE_RUNNING,
    }
    device_health = {
        "GOOD": Device.HEALTH_GOOD,
        "UNKNOWN": Device.HEALTH_UNKNOWN,
        "LOOPING": Device.HEALTH_LOOPING,
        "BAD": Device.HEALTH_BAD,
        "MAINTENANCE": Device.HEALTH_MAINTENANCE,
        "RETIRED": Device.HEALTH_RETIRED,
    }

    def add_arguments(self, parser):
        cmd = self

        class SubParser(CommandParser):
            """
            Sub-parsers constructor that mimic Django constructor.
            See http://stackoverflow.com/a/37414551
            """

            def __init__(self, **kwargs):
                super().__init__(cmd, **kwargs)

        sub = parser.add_subparsers(
            dest="sub_command", help="Sub commands", parser_class=SubParser
        )
        sub.required = True

        # "add" sub-command
        add_parser = sub.add_parser("add", help="Add a device")
        add_parser.add_argument("hostname", help="Hostname of the device")
        add_parser.add_argument("--device-type", required=True, help="Device type")
        add_parser.add_argument(
            "--description", default=None, help="Device description"
        )
        add_parser.add_argument(
            "--offline",
            action="store_false",
            dest="online",
            default=True,
            help="Create the device offline (online by default)",
        )
        add_parser.add_argument(
            "--private",
            action="store_false",
            dest="public",
            default=True,
            help="Make the device private (public by default)",
        )
        add_parser.add_argument(
            "--worker", required=True, help="The name of the worker"
        )
        add_parser.add_argument(
            "--tags",
            nargs="*",
            required=False,
            help="List of tags to add to the device",
        )
        physical = add_parser.add_mutually_exclusive_group()
        physical.add_argument(
            "--physical-user",
            help="Username of the user with physical access to the device",
        )
        physical.add_argument(
            "--physical-group",
            help="Name of the group with physical access to the device",
        )
        owner = add_parser.add_mutually_exclusive_group()
        owner.add_argument(
            "--owner", help="Username of the user with ownership of the device"
        )
        owner.add_argument(
            "--group", help="Name of the group with ownership of the device"
        )

        # "copy" sub-command
        add_parser = sub.add_parser(
            "copy", help="Copy an existing device as a new hostname"
        )
        add_parser.add_argument("original", help="Hostname of the existing device")
        add_parser.add_argument("target", help="Hostname of the device to create")
        add_parser.add_argument(
            "--offline",
            action="store_false",
            dest="online",
            default=True,
            help="Create the device offline (online by default)",
        )
        add_parser.add_argument(
            "--private",
            action="store_false",
            dest="public",
            default=True,
            help="Make the device private (public by default)",
        )
        add_parser.add_argument(
            "--worker", required=True, help="The name of the worker (required)"
        )
        add_parser.add_argument(
            "--copy-with-tags",
            action="store_true",
            dest="copytags",
            default=False,
            help="Set all the tags of the original device on the target device",
        )

        # "details" sub-command
        details_parser = sub.add_parser("details", help="Details about a device")
        details_parser.add_argument("hostname", help="Hostname of the device")

        # "list" sub-command
        list_parser = sub.add_parser("list", help="List the installed devices")
        list_parser.add_argument(
            "--state",
            default=None,
            choices=["IDLE", "RESERVED", "RUNNING"],
            help="Show only devices with the given state",
        )
        health = list_parser.add_mutually_exclusive_group()
        health.add_argument(
            "--all",
            "-a",
            dest="show_all",
            default=None,
            action="store_true",
            help="Show all devices, including retired ones",
        )
        health.add_argument(
            "--health",
            default=None,
            choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
            help="Show only devices with the given health",
        )
        list_parser.add_argument(
            "--csv", dest="csv", default=False, action="store_true", help="Print as csv"
        )

        # "update" sub-command
        update_parser = sub.add_parser(
            "update", help="Update properties of the given device"
        )
        update_parser.add_argument("hostname", help="Hostname of the device")
        update_parser.add_argument(
            "--description", default=None, help="Set the description"
        )
        update_parser.add_argument(
            "--health",
            default=None,
            choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
            help="Update the device health",
        )
        update_parser.add_argument("--worker", default=None, help="Update the worker")
        display = update_parser.add_mutually_exclusive_group()
        display.add_argument(
            "--public", default=None, action="store_true", help="make the device public"
        )
        display.add_argument(
            "--private",
            dest="public",
            action="store_false",
            help="Make the device private",
        )
        physical = update_parser.add_mutually_exclusive_group()
        physical.add_argument(
            "--physical-user",
            help="Username of the user with physical access to the device",
        )
        physical.add_argument(
            "--physical-group",
            help="Name of the group with physical access to the device",
        )
        owner = update_parser.add_mutually_exclusive_group()
        owner.add_argument(
            "--owner", help="Username of the user with ownership of the device"
        )
        owner.add_argument(
            "--group",
            dest="group",
            help="Name of the group with ownership of the device",
        )

    def handle(self, *args, **options):
        """ Forward to the right sub-handler """
        if options["sub_command"] == "add":
            self.handle_add(options)
        elif options["sub_command"] == "copy":
            self.handle_copy(options)
        elif options["sub_command"] == "details":
            self.handle_details(options["hostname"])
        elif options["sub_command"] == "list":
            self.handle_list(
                options["state"], options["health"], options["show_all"], options["csv"]
            )
        else:
            self.handle_update(options)

    def _assign(
        self, name, device, physical=False, owner=False, user=False, group=False
    ):
        if user:
            try:
                user = User.objects.get(username=name)
            except User.DoesNotExist:
                raise CommandError("Unable to find user '%s'" % name)
            if physical:
                if device.physical_group:
                    device.physical_group = None
                device.physical_owner = user
            elif owner:
                if device.group:
                    device.group = None
                device.user = user
            else:
                raise CommandError("Invalid combination of options.")
        elif group:
            try:
                group = Group.objects.get(name=name)
            except Group.DoesNotExist:
                raise CommandError("Unable to find group '%s'" % name)
            if physical:
                if device.physical_owner:
                    device.physical_owner = None
                device.physical_group = group
            elif owner:
                if device.user:
                    device.user = None
                device.group = group
            else:
                raise CommandError("Invalid combination of options.")
        else:
            raise CommandError("Invalid combination of options.")

    def handle_add(self, options):
        hostname = options["hostname"]
        device_type = options["device_type"]
        worker_name = options["worker"]
        description = options["description"]
        public = options["public"]
        online = options["online"]
        tags = options["tags"]

        with contextlib.suppress(Device.DoesNotExist):
            Device.objects.get(hostname=hostname)
            raise CommandError("Device '%s' already exists" % hostname)

        try:
            dt = DeviceType.objects.get(name=device_type)
        except DeviceType.DoesNotExist:
            raise CommandError("Unable to find device-type '%s'" % device_type)

        try:
            worker = Worker.objects.get(hostname=worker_name)
        except Worker.DoesNotExist:
            raise CommandError("Unable to find worker '%s'" % worker_name)

        health = Device.HEALTH_GOOD if online else Device.HEALTH_MAINTENANCE
        device = Device.objects.create(
            hostname=hostname,
            device_type=dt,
            description=description,
            state=Device.STATE_IDLE,
            health=health,
            worker_host=worker,
            is_public=public,
        )

        if tags is not None:
            for tag in tags:
                device.tags.add(Tag.objects.get_or_create(name=tag)[0])

        if options["physical_user"]:
            self._assign(options["physical_user"], device, user=True, physical=True)
        elif options["physical_group"]:
            self._assign(options["physical_group"], device, group=True, physical=True)

        if options["owner"]:
            self._assign(options["owner"], device, user=True, owner=True)
        elif options["group"]:
            self._assign(options["group"], device, group=True, owner=True)
        device.save()

    def handle_copy(self, options):
        original = options["original"]
        target = options["target"]
        worker_name = options["worker"]
        public = options["public"]
        online = options["online"]
        tags = options["copytags"]

        try:
            from_device = Device.objects.get(hostname=original)
        except Device.DoesNotExist:
            raise CommandError("Original device '%s' does NOT exist!" % original)

        with contextlib.suppress(Device.DoesNotExist):
            Device.objects.get(hostname=target)
            raise CommandError("Target device '%s' already exists" % target)

        origin = from_device.load_configuration()
        if not origin:
            raise CommandError("Device dictionary does not exist for %s" % original)

        if online:
            target_dict = os.path.join(Device.CONFIG_PATH, "%s.jinja2" % target)
            if not os.path.exists(target_dict):
                raise CommandError(
                    "Refusing to copy %s to new device %s with health 'Good' -"
                    " no device dictionary exists for target device, yet. "
                    "Use --offline or copy %s.jinja2 to "
                    "/etc/lava-server/dispatcher-config/devices/ and try again."
                    % (original, target, target)
                )

        try:
            worker = Worker.objects.get(hostname=worker_name)
        except Worker.DoesNotExist:
            raise CommandError("Unable to find worker '%s'" % worker_name)

        health = Device.HEALTH_GOOD if online else Device.HEALTH_MAINTENANCE
        description = from_device.description
        device_type = from_device.device_type
        from_tags = None
        if tags:
            from_tags = from_device.tags.all()

        physical_owner = None
        physical_group = None
        owner = None
        group = None

        if from_device.physical_owner:
            physical_owner = from_device.physical_owner
        elif from_device.physical_group:
            physical_group = from_device.physical_group

        if from_device.owner:
            owner = from_device.owner
        elif from_device.group:
            group = from_device.group

        with transaction.atomic():
            device = Device.objects.create(
                hostname=target,
                device_type=device_type,
                description=description,
                state=Device.STATE_IDLE,
                health=health,
                worker_host=worker,
                is_public=public,
            )

            if from_tags is not None:
                for tag in from_tags:
                    device.tags.add(tag)

            if physical_owner:
                self._assign(physical_owner, device, user=True, physical=True)
            elif physical_group:
                self._assign(physical_group, device, group=True, physical=True)

            if owner:
                self._assign(owner, device, user=True, owner=True)
            elif group:
                self._assign(group, device, group=True, owner=True)

            device.save()

        destiny = device.load_configuration()
        if not destiny:
            print("Reminder: device dictionary does not yet exist for %s" % target)

    def handle_details(self, hostname):
        """ Print device details """
        try:
            device = Device.objects.get(hostname=hostname)
        except Device.DoesNotExist:
            raise CommandError("Unable to find device '%s'" % hostname)

        self.stdout.write("hostname   : %s" % hostname)
        self.stdout.write("device-type: %s" % device.device_type.name)
        self.stdout.write("state      : %s" % device.get_state_display())
        self.stdout.write("health     : %s" % device.get_health_display())
        self.stdout.write("health job : %s" % bool(device.get_health_check()))
        self.stdout.write("description: %s" % device.description)
        self.stdout.write("public     : %s" % device.is_public)
        if device.user:
            owner = device.user.username
        elif device.group:
            owner = device.group.name
        else:
            owner = ""
        self.stdout.write("owner      : %s" % owner)

        if device.physical_owner:
            physical = device.physical_owner.username
        elif device.physical_group:
            physical = device.physical_group.name
        else:
            physical = ""
        self.stdout.write("physical   : %s" % physical)

        config = device.load_configuration(output_format="raw")
        self.stdout.write("device-dict: %s" % bool(config))
        self.stdout.write("worker     : %s" % device.worker_host.hostname)
        self.stdout.write("current_job: %s" % device.current_job())

    def handle_list(self, state, health, show_all, format_as_csv):
        """ Print devices list """
        devices = Device.objects.all().order_by("hostname")
        if state is not None:
            devices = devices.filter(state=self.device_state[state])

        if health is not None:
            devices = devices.filter(health=self.device_health[health])
        elif not show_all:
            devices = devices.exclude(health=Device.HEALTH_RETIRED)

        if format_as_csv:
            fields = ["hostname", "device-type", "state", "health"]
            writer = csv.DictWriter(self.stdout, fieldnames=fields)
            writer.writeheader()
            for device in devices:
                writer.writerow(
                    {
                        "hostname": device.hostname,
                        "device-type": device.device_type.name,
                        "state": device.get_state_display(),
                        "health": device.get_health_display(),
                    }
                )
        else:
            self.stdout.write("Available devices:")
            for device in devices:
                self.stdout.write(
                    "* %s (%s) %s, %s"
                    % (
                        device.hostname,
                        device.device_type.name,
                        device.get_state_display(),
                        device.get_health_display(),
                    )
                )

    def handle_update(self, options):
        """ Update device properties """
        with transaction.atomic():
            hostname = options["hostname"]
            try:
                device = Device.objects.select_for_update().get(hostname=hostname)
            except Device.DoesNotExist:
                raise CommandError("Unable to find device '%s'" % hostname)

            health = options["health"]
            if health is not None:
                device.health = self.device_health[health]

            description = options["description"]
            if description is not None:
                device.description = description

            worker_name = options["worker"]
            if worker_name is not None:
                try:
                    worker = Worker.objects.get(hostname=worker_name)
                    device.worker_host = worker
                except Worker.DoesNotExist:
                    raise CommandError("Unable to find worker '%s'" % worker_name)

            public = options["public"]
            if public is not None:
                device.is_public = public

            if options["physical_user"]:
                self._assign(options["physical_user"], device, user=True, physical=True)
            elif options["physical_group"]:
                self._assign(
                    options["physical_group"], device, group=True, physical=True
                )

            if options["owner"]:
                self._assign(options["owner"], device, user=True, owner=True)
            elif options["group"]:
                self._assign(options["group"], device, group=True, owner=True)

            # Save the modifications
            device.save()



# Copyright (C) 2015 Linaro Limited
#
# Author: Stevan Radakovic <stevan.radakovic@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

import contextlib
import os
import yaml

from lava_dispatcher.action import Action
from lava_common.constants import LINE_SEPARATOR


class DeployDeviceEnvironment(Action):
    """
    Create environment found in job parameters 'env_dut' and set it in common_data.
    """

    name = "deploy-device-env"
    description = "deploy device environment"
    summary = "deploy device environment"

    def __init__(self):
        super().__init__()
        self.env = ""

    def validate(self):
        super().validate()
        shell_file = self.get_constant("lava_test_shell_file", "posix")
        if not shell_file:
            self.errors = "Invalid deployment data - missing lava_test_shell_file"

        if "env_dut" in self.job.parameters and self.job.parameters["env_dut"]:
            # Check that the file is valid yaml
            try:
                yaml.safe_load(self.job.parameters["env_dut"])
            except (TypeError, yaml.scanner.ScannerError) as exc:
                self.errors = exc
                return

            self.env = self.job.parameters["env_dut"]
            environment = self._create_environment()

            self.set_namespace_data(
                action=self.name,
                label="environment",
                key="shell_file",
                value=shell_file,
            )

            self.set_namespace_data(
                action=self.name, label="environment", key="env_dict", value=environment
            )

        self.set_namespace_data(
            action=self.name,
            label="environment",
            key="line_separator",
            value=self.parameters["deployment_data"].get(
                "line_separator", LINE_SEPARATOR
            ),
        )

    def _create_environment(self):
        """Generate the env variables for the device."""
        conf = yaml.safe_load(self.env) if self.env != "" else {}
        if conf.get("purge", False):
            environ = {}
        else:
            environ = dict(os.environ)

        # Remove some variables (that might not exist)
        for var in conf.get("removes", {}):
            with contextlib.suppress(KeyError):
                del environ[var]

        # Override
        environ.update(conf.get("overrides", {}))
        return environ



# -*- coding: utf-8 -*-
# Copyright (C) 2015-2018 Linaro Limited
#
# Author: Stevan Radakovic <stevan.radakovic@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from lava_results_app.models import Chart, ChartQuery, ChartQueryUser, TestCase


class ChartForm(forms.ModelForm):
    class Meta:
        model = Chart
        exclude = ("is_published", "chart_group", "group", "queries")
        widgets = {"owner": forms.HiddenInput}

    def __init__(self, owner, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save(self, commit=True, **kwargs):
        instance = super().save(commit=commit, **kwargs)
        return instance


class ChartQueryForm(forms.ModelForm):
    class Meta:
        model = ChartQuery
        exclude = ()
        widgets = {
            "chart": forms.HiddenInput,
            "query": forms.HiddenInput,
            "relative_index": forms.HiddenInput,
        }

    def __init__(self, user, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save(self, commit=True, **kwargs):
        instance = super().save(commit=commit, **kwargs)
        return instance

    def clean(self):
        form_data = self.cleaned_data

        try:
            # Chart type validation.
            if (
                form_data["query"].content_type.model_class() == TestCase
                and form_data["chart_type"] == "pass/fail"
            ):
                self.add_error(
                    "chart_type",
                    "Pass/fail is incorrect value for 'chart_type' with TestCase based queries.",
                )

        except KeyError:
            # form_data will pick up the rest of validation errors.
            pass

        return form_data


class ChartQueryUserForm(forms.ModelForm):
    class Meta:
        model = ChartQueryUser
        exclude = ["user", "chart_query"]

    def __init__(self, user, *args, **kwargs):
        super().__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
from django.db import migrations, models
import django.db.models.deletion
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0009_testjob_pipeline_compatibility")]

    operations = [
        migrations.AlterField(
            model_name="device",
            name="current_job",
            field=models.OneToOneField(
                related_name="+",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                blank=True,
                to="lava_scheduler_app.TestJob",
            ),
        ),
        migrations.AlterField(
            model_name="device",
            name="last_health_report_job",
            field=models.OneToOneField(
                related_name="+",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                blank=True,
                to="lava_scheduler_app.TestJob",
            ),
        ),
        migrations.AlterField(
            model_name="device",
            name="physical_group",
            field=models.ForeignKey(
                related_name="physicalgroup",
                default=None,
                blank=True,
                to="auth.Group",
                null=True,
                verbose_name="Group with physical access",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterField(
            model_name="device",
            name="physical_owner",
            field=models.ForeignKey(
                related_name="physicalowner",
                default=None,
                blank=True,
                to=settings.AUTH_USER_MODEL,
                null=True,
                verbose_name="User with physical access",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterField(
            model_name="devicetype",
            name="cores",
            field=models.ManyToManyField(
                related_name="device_types", to="lava_scheduler_app.Core", blank=True
            ),
        ),
        migrations.AlterField(
            model_name="testjob",
            name="viewing_groups",
            field=models.ManyToManyField(
                related_name="viewing_groups",
                default=None,
                to="auth.Group",
                blank=True,
                help_text="Adding groups to an intersection of groups reduces visibility.Adding groups to a union of groups expands visibility.",
                verbose_name="Viewing groups",
            ),
        ),
    ]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
zmq_client.py script
"""
#  Copyright 2018 Linaro
#  Author: Neil Williams <neil.williams@linaro.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=missing-docstring,no-member,unused-variable,too-few-public-methods
# pylint: disable=invalid-name,unused-argument,no-self-use,wrong-import-order
# START_CLIENT

import sys
import ssl
import argparse
import yaml
import signal
import zmq
import xmlrpc.client
from urllib.parse import urlsplit


FINISHED_JOB_STATUS = ["Complete", "Incomplete", "Canceled"]


class JobEndTimeoutError(Exception):
    """ Raise when the specified job does not finish in certain timeframe. """


class Timeout:
    """ Timeout error class with ALARM signal. Accepts time in seconds. """

    class TimeoutError(Exception):
        pass

    def __init__(self, sec=0):
        self.sec = sec

    def __enter__(self):
        signal.signal(signal.SIGALRM, self.timeout_raise)
        if not self.sec:
            self.sec = 0
        signal.alarm(self.sec)

    def __exit__(self, *args):
        signal.alarm(0)

    def timeout_raise(self, *args):
        raise Timeout.TimeoutError()


class JobListener:
    def __init__(self, url):
        self.context = zmq.Context.instance()
        self.sock = self.context.socket(zmq.SUB)

        self.sock.setsockopt(zmq.SUBSCRIBE, b"")
        self.sock.connect(url)

    def wait_for_job_end(self, job_id, timeout=None):

        try:
            with Timeout(timeout):
                while True:
                    msg = self.sock.recv_multipart()
                    try:
                        (topic, uuid, dt, username, data) = msg[
                            :
                        ]  # pylint: disable=unbalanced-tuple-unpacking
                    except IndexError:
                        # Droping invalid message
                        continue

                    data = yaml.safe_load(data)
                    if "job" in data:
                        if data["job"] == job_id:
                            if data["health"] in FINISHED_JOB_STATUS:
                                return data

        except Timeout.TimeoutError:
            raise JobEndTimeoutError(
                "JobListener timed out after %s seconds." % timeout
            )


def lookup_publisher(hostname, https):
    """
    Lookup the publisher details using XML-RPC
    on the specified hostname.
    """
    xmlrpc_url = "http://%s/RPC2" % (hostname)
    if https:
        xmlrpc_url = "https://%s/RPC2" % (hostname)
    server = xmlrpc.client.ServerProxy(xmlrpc_url)
    try:
        socket = server.scheduler.get_publisher_event_socket()
    except ssl.SSLError as exc:
        sys.stderr.write("ERROR %s\n" % exc)
        return None
    port = urlsplit(socket).port
    listener_url = "tcp://%s:%s" % (hostname, port)
    print("Using %s" % listener_url)
    return listener_url


def main():
    """
    Parse the command line
    For simplicity, this script does not handle usernames
    and tokens so needs a job ID. For support submitting
    a test job as well as watching the events, use lava-tool.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-j", "--job-id", type=int, help="Job ID to wait for")

    parser.add_argument(
        "--https", action="store_true", help="Use https:// for this hostname"
    )

    parser.add_argument("-t", "--timeout", type=int, help="Timeout in seconds")
    parser.add_argument("--hostname", required=True, help="hostname of the instance")

    options = parser.parse_args()

    try:
        publisher = lookup_publisher(options.hostname, options.https)
    except xmlrpc.client.ProtocolError as exc:
        sys.stderr.write("ERROR %s\n" % exc)
        return 1

    if not publisher:
        return 1

    if options.job_id:
        listener = JobListener(publisher)
        print(listener.wait_for_job_end(options.job_id, options.timeout))
    print("\n")
    return 0


if __name__ == "__main__":
    main()

# END_CLIENT



# pylint: disable=ungrouped-imports
import os
import re
import yaml
import shutil
import logging
import decimal

import django
from django.core.exceptions import MultipleObjectsReturned
from django.urls.exceptions import NoReverseMatch
from django.urls import reverse

from lava_results_app.tests.test_names import TestCaseWithFactory
from lava_scheduler_app.models import TestJob, Device
from lava_scheduler_app.utils import mkdir
from lava_results_app.dbutils import (
    map_metadata,
    map_scanned_results,
    create_metadata_store,
    _get_action_metadata,  # pylint: disable=protected-access
)
from lava_results_app.models import ActionData, MetaType, TestData, TestCase, TestSuite
from lava_results_app.utils import export_testcase, testcase_export_fields
from lava_dispatcher.parser import JobParser
from lava_dispatcher.device import PipelineDevice
from lava_dispatcher.tests.test_defs import allow_missing_path

# pylint: disable=invalid-name,too-few-public-methods,too-many-public-methods,no-member,too-many-ancestors


class TestMetaTypes(TestCaseWithFactory):
    """
    MetaType and ActionData generation
    """

    def setUp(self):
        super().setUp()
        logger = logging.getLogger("lava-master")
        logger.disabled = True

    def test_job(self):
        MetaType.objects.all().delete()
        TestJob.objects.all().delete()
        job = TestJob.from_yaml_and_user(self.factory.make_job_yaml(), self.user)
        job_def = yaml.safe_load(job.definition)
        job_ctx = job_def.get("context", {})
        job_ctx.update(
            {"no_kvm": True}
        )  # override to allow unit tests on all types of systems
        device = Device.objects.get(hostname="fakeqemu1")
        device_config = device.load_configuration(job_ctx)  # raw dict
        parser = JobParser()
        obj = PipelineDevice(device_config)
        pipeline_job = parser.parse(job.definition, obj, job.id, None, "")
        allow_missing_path(
            pipeline_job.pipeline.validate_actions, self, "qemu-system-x86_64"
        )
        pipeline = pipeline_job.describe()
        map_metadata(yaml.dump(pipeline), job)
        self.assertEqual(
            MetaType.objects.filter(metatype=MetaType.DEPLOY_TYPE).count(), 1
        )
        self.assertEqual(
            MetaType.objects.filter(metatype=MetaType.BOOT_TYPE).count(), 1
        )
        count = ActionData.objects.all().count()
        self.assertEqual(TestData.objects.all().count(), 1)
        testdata = TestData.objects.all()[0]
        self.assertEqual(testdata.testjob, job)
        for actionlevel in ActionData.objects.all():
            self.assertEqual(actionlevel.testdata, testdata)
        action_levels = []
        for testdata in job.testdata_set.all():
            action_levels.extend(testdata.actionlevels.all())
        self.assertEqual(count, len(action_levels))
        count = ActionData.objects.filter(
            meta_type__metatype=MetaType.DEPLOY_TYPE
        ).count()
        self.assertNotEqual(
            ActionData.objects.filter(meta_type__metatype=MetaType.BOOT_TYPE).count(), 0
        )
        self.assertEqual(
            ActionData.objects.filter(
                meta_type__metatype=MetaType.UNKNOWN_TYPE
            ).count(),
            0,
        )
        for actionlevel in ActionData.objects.filter(
            meta_type__metatype=MetaType.BOOT_TYPE
        ):
            self.assertEqual(actionlevel.testdata.testjob.id, job.id)
        self.assertEqual(
            ActionData.objects.filter(
                meta_type__metatype=MetaType.DEPLOY_TYPE, testdata__testjob=job
            ).count(),
            count,
        )

    def test_export(self):
        job = TestJob.from_yaml_and_user(self.factory.make_job_yaml(), self.user)
        test_suite = TestSuite.objects.get_or_create(name="lava", job=job)[0]
        test_case = TestCase(
            id=1, name="name", suite=test_suite, result=TestCase.RESULT_FAIL
        )
        self.assertTrue(
            any(
                map(
                    lambda v: v in testcase_export_fields(),
                    export_testcase(test_case).keys(),
                )
            )
        )

    def test_duration(self):
        TestJob.from_yaml_and_user(self.factory.make_job_yaml(), self.user)
        metatype = MetaType(name="fake", metatype=MetaType.DEPLOY_TYPE)
        metatype.save()
        action_data = ActionData(
            meta_type=metatype, action_level="1.2.3", action_name="fake"
        )
        action_data.save()
        action_data.duration = "1.2"
        action_data.save(update_fields=["duration"])
        action_data = ActionData.objects.get(id=action_data.id)  # reload
        self.assertIsInstance(action_data.duration, decimal.Decimal)
        # unit tests check the instance as well as the value.
        self.assertEqual(float(action_data.duration), 1.2)
        action_data.timeout = 300
        action_data.save(update_fields=["timeout"])
        self.assertEqual(action_data.timeout, 300)

    def test_decimal_yaml_dump(self):
        job = TestJob.from_yaml_and_user(self.factory.make_job_yaml(), self.user)
        test_dict = {
            "definition": "unit-test",
            "case": "unit-test",
            "measurement": decimal.Decimal(1234.5),
            "result": "pass",
        }
        test_case = map_scanned_results(test_dict, job, {}, None)
        metadata_yaml_ref = "{case: unit-test, definition: unit-test, measurement: '1234.5', result: pass}"
        self.assertEqual(metadata_yaml_ref, test_case.metadata.strip())

    def test_case_as_url(self):
        job = TestJob.from_yaml_and_user(self.factory.make_job_yaml(), self.user)
        test_dict = {
            "definition": "unit-test",
            "case": "unit-test",
            "level": "1.3.4.1",
            # list of numbers, generates a much longer YAML string than just the count
            "result": "pass",
        }
        pattern = "[-_a-zA-Z0-9.\\(\\)]+"
        matches = re.search(pattern, test_dict["case"])
        self.assertIsNotNone(matches)  # passes
        self.assertEqual(matches.group(0), test_dict["case"])
        suite, _ = TestSuite.objects.get_or_create(
            name=test_dict["definition"], job=job
        )
        case, _ = TestCase.objects.get_or_create(
            suite=suite, name=test_dict["case"], result=TestCase.RESULT_PASS
        )
        self.assertIsNotNone(reverse("lava.results.testcase", args=[case.id]))
        self.assertIsNotNone(
            reverse("lava.results.testcase", args=[job.id, suite.name, case.id])
        )
        self.assertIsNotNone(map_scanned_results(test_dict, job, {}, None))
        # now break the reverse pattern
        test_dict["case"] = "unit test"  # whitespace in the case name
        matches = re.search(pattern, test_dict["case"])
        self.assertIsNotNone(matches)
        self.assertRaises(
            NoReverseMatch,
            reverse,
            "lava.results.testcase",
            args=[job.id, suite.name, test_dict["case"]],
        )

    def test_metastore(self):
        field = TestCase._meta.get_field("metadata")
        level = "1.3.5.1"
        # artificially inflate results to represent a set of kernel messages
        results = {
            "definition": "lava",
            "case": "unit-test",
            "level": level,
            # list of numbers, generates a much longer YAML string than just the count
            "extra": range(int(field.max_length / 2)),
            "result": "pass",
        }
        stub = "%s-%s-%s.yaml" % (results["definition"], results["case"], level)
        job = TestJob.from_yaml_and_user(self.factory.make_job_yaml(), self.user)
        meta_filename = os.path.join(job.output_dir, "metadata", stub)
        filename = "%s/job-%s/pipeline/%s/%s-%s.yaml" % (
            job.output_dir,
            job.id,
            level.split(".")[0],
            level,
            results["definition"],
        )

        mkdir(os.path.dirname(filename))
        if os.path.exists(meta_filename):
            # isolate from other unit tests
            os.unlink(meta_filename)
        self.assertEqual(meta_filename, create_metadata_store(results, job))
        ret = map_scanned_results(results, job, {}, meta_filename)
        self.assertIsNotNone(ret)
        ret.save()
        self.assertEqual(TestCase.objects.filter(name="unit-test").count(), 1)
        test_data = yaml.load(  # nosec - unit test
            TestCase.objects.filter(name="unit-test")[0].metadata, Loader=yaml.CLoader
        )
        self.assertEqual(test_data["extra"], meta_filename)
        self.assertTrue(os.path.exists(meta_filename))
        with open(test_data["extra"], "r") as extra_file:
            data = yaml.load(extra_file, Loader=yaml.CLoader)  # nosec - unit test
        self.assertIsNotNone(data)
        os.unlink(meta_filename)
        shutil.rmtree(job.output_dir)

    def test_repositories(self):  # pylint: disable=too-many-locals
        job = TestJob.from_yaml_and_user(self.factory.make_job_yaml(), self.user)
        job_def = yaml.safe_load(job.definition)
        job_ctx = job_def.get("context", {})
        job_ctx.update(
            {"no_kvm": True}
        )  # override to allow unit tests on all types of systems
        device = Device.objects.get(hostname="fakeqemu1")
        device_config = device.load_configuration(job_ctx)  # raw dict
        parser = JobParser()
        obj = PipelineDevice(device_config)
        pipeline_job = parser.parse(job.definition, obj, job.id, None, "")
        allow_missing_path(
            pipeline_job.pipeline.validate_actions, self, "qemu-system-x86_64"
        )
        pipeline = pipeline_job.describe()
        try:
            testdata, _ = TestData.objects.get_or_create(testjob=job)
        except (MultipleObjectsReturned):
            self.fail("multiple objects")
            testdata.attributes.create(name=key, value=value)
        retval = _get_action_metadata(pipeline["job"]["actions"])
        self.assertEqual(
            retval,
            {
                "test.1.common.definition.from": "git",
                "test.0.common.definition.repository": "git://git.linaro.org/lava-team/lava-functional-tests.git",
                "test.0.common.definition.name": "smoke-tests",
                "test.1.common.definition.repository": "http://git.linaro.org/lava-team/lava-functional-tests.git",
                "boot.0.common.method": "qemu",
                "test.1.common.definition.name": "singlenode-advanced",
                "test.0.common.definition.from": "git",
                "test.0.common.definition.path": "lava-test-shell/smoke-tests-basic.yaml",
                "test.1.common.definition.path": "lava-test-shell/single-node/singlenode03.yaml",
            },
        )

    def test_parameter_support(self):  # pylint: disable=too-many-locals
        data = self.factory.make_job_data()
        test_block = [block for block in data["actions"] if "test" in block][0]
        smoke = test_block["test"]["definitions"][0]
        smoke["parameters"] = {
            "VARIABLE_NAME_1": "first variable value",
            "VARIABLE_NAME_2": "second value",
        }
        job = TestJob.from_yaml_and_user(yaml.dump(data), self.user)
        job_def = yaml.safe_load(job.definition)
        job_ctx = job_def.get("context", {})
        job_ctx.update(
            {"no_kvm": True}
        )  # override to allow unit tests on all types of systems
        device = Device.objects.get(hostname="fakeqemu1")
        device_config = device.load_configuration(job_ctx)  # raw dict
        parser = JobParser()
        obj = PipelineDevice(device_config)
        pipeline_job = parser.parse(job.definition, obj, job.id, None, "")
        allow_missing_path(
            pipeline_job.pipeline.validate_actions, self, "qemu-system-x86_64"
        )
        pipeline = pipeline_job.describe()
        try:
            testdata, _ = TestData.objects.get_or_create(testjob=job)
        except (MultipleObjectsReturned):
            self.fail("multiple objects")
            testdata.attributes.create(name=key, value=value)
        retval = _get_action_metadata(pipeline["job"]["actions"])
        self.assertIn("test.0.common.definition.parameters.VARIABLE_NAME_2", retval)
        self.assertIn("test.0.common.definition.parameters.VARIABLE_NAME_1", retval)
        self.assertEqual(
            retval["test.0.common.definition.parameters.VARIABLE_NAME_1"],
            "first variable value",
        )
        self.assertEqual(
            retval["test.0.common.definition.parameters.VARIABLE_NAME_2"],
            "second value",
        )

    def test_job_multi(self):
        MetaType.objects.all().delete()
        multi_test_file = os.path.join(os.path.dirname(__file__), "multi-test.yaml")
        self.assertTrue(os.path.exists(multi_test_file))
        with open(multi_test_file, "r") as test_support:
            data = test_support.read()
        job = TestJob.from_yaml_and_user(data, self.user)
        job_def = yaml.safe_load(job.definition)
        job_ctx = job_def.get("context", {})
        job_ctx.update(
            {"no_kvm": True}
        )  # override to allow unit tests on all types of systems
        device = Device.objects.get(hostname="fakeqemu1")
        device_config = device.load_configuration(job_ctx)  # raw dict
        parser = JobParser()
        obj = PipelineDevice(device_config)
        pipeline_job = parser.parse(job.definition, obj, job.id, None, "")
        allow_missing_path(
            pipeline_job.pipeline.validate_actions, self, "qemu-system-x86_64"
        )
        pipeline = pipeline_job.describe()
        map_metadata(yaml.dump(pipeline), job)

    def test_inline(self):
        """
        Test inline can be parsed without run steps
        """
        data = self.factory.make_job_data()
        test_block = [block for block in data["actions"] if "test" in block][0]
        smoke = [
            {
                "path": "inline/smoke-tests-basic.yaml",
                "from": "inline",
                "name": "smoke-tests-inline",
                "repository": {
                    "install": {"steps": ["apt"]},
                    "metadata": {
                        "description": "Basic system test command for Linaro Ubuntu images",
                        "format": "Lava-Test Test Definition 1.0",
                        "name": "smoke-tests-basic",
                    },
                },
            }
        ]
        test_block["test"]["definitions"] = smoke
        job = TestJob.from_yaml_and_user(yaml.dump(data), self.user)
        job_def = yaml.safe_load(job.definition)
        job_ctx = job_def.get("context", {})
        job_ctx.update(
            {"no_kvm": True}
        )  # override to allow unit tests on all types of systems
        device = Device.objects.get(hostname="fakeqemu1")
        device_config = device.load_configuration(job_ctx)  # raw dict
        parser = JobParser()
        obj = PipelineDevice(device_config)
        pipeline_job = parser.parse(job.definition, obj, job.id, None, "")
        allow_missing_path(
            pipeline_job.pipeline.validate_actions, self, "qemu-system-x86_64"
        )
        pipeline = pipeline_job.describe()
        map_metadata(yaml.dump(pipeline), job)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dashboard_app', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bundle',
            name='is_deserialized',
            field=models.BooleanField(default=False, help_text='Set when document has been analyzed and loaded into the database', verbose_name='Is deserialized', editable=False),
        ),
        migrations.AlterField(
            model_name='bundlestream',
            name='is_anonymous',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='testrun',
            name='time_check_performed',
            field=models.BooleanField(default=False, help_text="Indicator on wether timestamps in the log file (and any data derived from them) should be trusted.<br/>Many pre-production or development devices do not have a battery-powered RTC and it's not common for development images not to synchronize time with internet time servers.<br/>This field allows us to track tests results that <em>certainly</em> have correct time if we ever end up with lots of tests results from 1972", verbose_name='Time check performed'),
        ),
    ]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import glob
import fnmatch
from setuptools import setup, find_packages
from version import version_tag


# based on https://wiki.python.org/moin/Distutils/Tutorial
def find_data_files(srcdir, *wildcards):
    badnames = [".pyc", "~"]

    def walk_helper(arg, dirname, files):
        names = []
        lst, wilds = arg
        for wcard in wilds:
            wc_name = os.path.normpath(os.path.join(dirname, wcard))
            for listed in files:
                filename = os.path.normpath(os.path.join(dirname, listed))
                if not any(bad in filename for bad in badnames):
                    if fnmatch.fnmatch(filename, wc_name) and not os.path.isdir(
                        filename
                    ):
                        names.append(filename)
        if names:
            lst.append(("/etc/lava-server/dispatcher-config/device-types", names))

    file_list = []
    walk_helper(
        (file_list, wildcards),
        srcdir,
        [
            os.path.basename(f)
            for f in glob.glob(os.path.normpath(os.path.join(srcdir, "*")))
        ],
    )
    return file_list


SRCDIR = os.path.join(".", "lava_scheduler_app", "tests", "device-types")
DEVICE_TYPE_TEMPLATES = find_data_files(SRCDIR, "*.jinja2")

setup(
    name="lava",
    version=version_tag(),
    author="Neil Williams",
    author_email="lava-team@linaro.org",
    namespace_packages=["lava"],
    packages=find_packages(),
    test_suite="lava_server.tests.run_tests",
    license="AGPL",
    description="LAVA",
    long_description="""
     LAVA is a continuous integration system for deploying operating
     systems onto physical and virtual hardware for running tests.
     Tests can be simple boot testing, bootloader testing and system
     level testing. Extra hardware may be required for some
     system tests. Results are tracked over time and data can be
    exported for further analysis.
    """,
    url="https://www.lavasoftware.org/",
    package_data={
        "lava_dispatcher": [
            "dynamic_vm_keys/lava*",
            "devices/*.yaml",
            "lava_test_shell/lava-add-keys",
            "lava_test_shell/lava-add-sources",
            "lava_test_shell/lava-background-process-start",
            "lava_test_shell/lava-background-process-stop",
            "lava_test_shell/lava-echo-ipv4",
            "lava_test_shell/lava-installed-packages",
            "lava_test_shell/lava-install-packages",
            "lava_test_shell/lava-lxc-device-add",
            "lava_test_shell/lava-lxc-device-wait-add",
            "lava_test_shell/lava-os-build",
            "lava_test_shell/lava-probe-channel",
            "lava_test_shell/lava-probe-ip",
            "lava_test_shell/lava-target-ip",
            "lava_test_shell/lava-target-mac",
            "lava_test_shell/lava-target-storage",
            "lava_test_shell/lava-test-case",
            "lava_test_shell/lava-test-feedback",
            "lava_test_shell/lava-test-raise",
            "lava_test_shell/lava-test-reference",
            "lava_test_shell/lava-test-runner",
            "lava_test_shell/lava-test-set",
            "lava_test_shell/lava-test-shell",
            "lava_test_shell/multi_node/*",
            "lava_test_shell/vland/*",
            "lava_test_shell/lmp/*",
            "lava_test_shell/distro/fedora/*",
            "lava_test_shell/distro/android/*",
            "lava_test_shell/distro/ubuntu/*",
            "lava_test_shell/distro/debian/*",
            "lava_test_shell/distro/oe/*",
        ]
    },
    scripts=["lava/dispatcher/lava-run", "lava/dispatcher/lava-slave"],
    data_files=[
        ("/usr/share/lava-dispatcher/", ["etc/tftpd-hpa", "etc/dispatcher.yaml"]),
        ("/etc/exports.d", ["etc/lava-dispatcher-nfs.exports"]),
        ("/etc/modprobe.d/", ["etc/lava-modules.conf"]),
        ("/etc/logrotate.d/", ["etc/logrotate.d/lava-slave-log"]),
        ("/usr/share/lava-dispatcher/", ["etc/lava-slave.service"]),
        ("/etc/lava-server", ["etc/settings.conf", "etc/env.yaml"]),
        ("/etc/apache2/sites-available", ["etc/lava-server.conf"]),
        (
            "/etc/logrotate.d",
            [
                "etc/logrotate.d/django-log",
                "etc/logrotate.d/lava-master-log",
                "etc/logrotate.d/lava-publisher-log",
                "etc/logrotate.d/lava-server-gunicorn-log",
            ],
        ),
        (
            "/usr/share/lava-server",
            [
                "etc/lava-master.service",
                "etc/lava-publisher.service",
                "etc/lava-logs.service",
                "etc/instance.conf.template",
                "share/render-template.py",
            ],
        ),
    ].extend(DEVICE_TYPE_TEMPLATES),
    tests_require=["django-testscenarios >= 0.9.1"],
    zip_safe=False,
    include_package_data=True,
)



# Copyright (C) 2017 The Linux Foundation
#
# Author: Jan-Simon Moeller <jsmoeller@linuxfoundation.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

# List just the subclasses supported for this base strategy
# imported by the parser to populate the list of subclasses.

import os
import tempfile

from lava_dispatcher.action import Pipeline
from lava_common.exceptions import JobError
from lava_dispatcher.logical import Deployment
from lava_dispatcher.actions.deploy import DeployAction
from lava_dispatcher.actions.deploy.download import DownloaderAction
from lava_dispatcher.utils.shell import which
from lava_dispatcher.utils.filesystem import tftpd_dir
from lava_dispatcher.protocols.xnbd import XnbdProtocol
from lava_dispatcher.actions.deploy.overlay import OverlayAction


class Nbd(Deployment):
    """
    Strategy class for a tftp+initrd+nbd based Deployment.
    tftp is used for kernel/initrd/fdt. Rootfs over nbd (network block device).
    Downloads the relevant parts, copies to the tftp location.
    Limited to what the bootloader can deploy which means ramdisk or nfsrootfs.
    rootfs deployments would format the device and create a single partition for the rootfs.
    """

    compatibility = 1
    name = "nbd"

    def __init__(self, parent, parameters):
        super().__init__(parent)
        self.action = NbdAction()
        self.action.section = self.action_type
        self.action.job = self.job
        parent.add_action(self.action, parameters)

    @classmethod
    def accepts(cls, device, parameters):
        if "to" not in parameters:
            return False, '"to" is not in deploy parameters'
        if parameters["to"] != "nbd":
            return False, '"to" parameter is not "nbd"'
        if "nbd" in device["actions"]["deploy"]["methods"]:
            return True, "accepted"
        return False, '"ndb" was not in the device configuration deploy methods'


class NbdAction(DeployAction):  # pylint:disable=too-many-instance-attributes

    name = "nbd-deploy"
    description = "download files and deploy for using tftp+initrd+nbd"
    summary = "nbd deployment"

    def __init__(self):
        super().__init__()
        self.tftp_dir = None
        self.nbd_ip = None
        self.nbd_port = None

    def validate(self):
        super().validate()
        if "kernel" not in self.parameters:
            self.errors = "%s needs a kernel to deploy" % self.name
        if not self.valid:
            return
        if "nbdroot" not in self.parameters:
            self.errors = "NBD deployment needs a 'nbdroot' parameter"
        if "initrd" not in self.parameters:
            self.errors = "NBD deployment needs an 'initrd' parameter"
        # we cannot work with these when using nbd
        if "nfsrootfs" in self.parameters:
            self.errors = "nfsrootfs cannot be used with NBD deployment, use a e.g. ext3/4 filesystem as 'nbdroot=' parameter"
        if "ramdisk" in self.parameters:
            self.errors = "ramdisk cannot be used with NBD deployment, use a e.g. ext3/4 filesystem as 'initrd' parameter"

        # Extract the 3 last path elements. See action.mkdtemp()
        suffix = os.path.join(*self.tftp_dir.split("/")[-2:])
        self.set_namespace_data(
            action="tftp-deploy", label="tftp", key="suffix", value=suffix
        )
        # we need tftp _and_ xnbd-server
        which("in.tftpd")
        which("xnbd-server")

        # Check that the tmp directory is in the nbdd_dir or in /tmp for the
        # unit tests
        tftpd_directory = os.path.realpath(tftpd_dir())
        tftp_dir = os.path.realpath(self.tftp_dir)
        tmp_dir = tempfile.gettempdir()
        if not tftp_dir.startswith(tftpd_directory) and not tftp_dir.startswith(
            tmp_dir
        ):
            self.errors = "tftpd directory is not configured correctly, see /etc/default/tftpd-hpa"

    def populate(self, parameters):
        self.tftp_dir = self.mkdtemp(override=tftpd_dir())
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        self.set_namespace_data(
            action=self.name,
            label="tftp",
            key="tftp_dir",
            value=self.tftp_dir,
            parameters=parameters,
        )

        for key in ["initrd", "kernel", "dtb", "nbdroot"]:
            if key in parameters:
                download = DownloaderAction(key, path=self.tftp_dir)
                download.max_retries = (
                    3
                )  # overridden by failure_retry in the parameters, if set.
                self.internal_pipeline.add_action(download)
                if key == "initrd":
                    self.set_namespace_data(
                        action="tftp-deploy",
                        label="tftp",
                        key="ramdisk",
                        value=True,
                        parameters=parameters,
                    )
                    self.set_namespace_data(
                        action=self.name,
                        label="nbd",
                        key="initrd",
                        value=True,
                        parameters=parameters,
                    )

        # prepare overlay
        self.internal_pipeline.add_action(OverlayAction())
        # setup values for protocol and later steps
        self.set_namespace_data(
            action=self.name,
            label="nbd",
            key="initrd",
            value=True,
            parameters=parameters,
        )
        # store in parameters for protocol 'xnbd' to tear-down xnbd-server
        # and store in namespace for boot action
        # ip
        parameters["lava-xnbd"] = {}
        # handle XnbdAction next - bring-up xnbd-server
        self.internal_pipeline.add_action(XnbdAction())


class XnbdAction(DeployAction):

    name = "xnbd-server-deploy"
    description = "xnbd daemon"
    summary = "xnbd daemon"

    def __init__(self):
        super().__init__()
        self.protocol = XnbdProtocol.name
        self.nbd_server_port = None
        self.nbd_server_ip = None

    def run(self, connection, max_end_time):
        connection = super().run(connection, max_end_time)
        self.logger.debug("%s: starting xnbd-server", self.name)
        # pull from parameters - as previously set
        self.nbd_root = self.parameters["lava-xnbd"]["nbdroot"]
        self.nbd_server_port = self.get_namespace_data(
            action="nbd-deploy", label="nbd", key="nbd_server_port"
        )
        self.nbd_server_ip = self.get_namespace_data(
            action="nbd-deploy", label="nbd", key="nbd_server_ip"
        )
        if self.nbd_server_port is None:
            self.errors = "NBD server port is unset"
            return connection
        self.logger.debug(
            "NBD-IP: %s, NBD-PORT: %s, NBD-ROOT: %s",
            self.nbd_server_ip,
            self.nbd_server_port,
            self.nbd_root,
        )
        nbd_cmd = [
            "xnbd-server",
            "--logpath",
            "/tmp/xnbd.log.%s" % self.nbd_server_port,
            "--daemon",
            "--target",
            "--lport",
            "%s" % self.nbd_server_port,
            "%s/%s" % (os.path.realpath(tftpd_dir()), self.nbd_root),
        ]
        command_output = self.run_command(nbd_cmd, allow_fail=False)

        if command_output and "error" in command_output:
            raise JobError("xnbd-server: %s" % command_output)
        else:
            self.logger.debug("%s: starting xnbd-server done", self.name)
        return connection



# pylint: disable=superfluous-parens,ungrouped-imports
from lava_scheduler_app.tests.test_base_templates import (
    BaseTemplate,
    prepare_jinja_template,
)

# pylint: disable=too-many-branches,too-many-public-methods
# pylint: disable=too-many-nested-blocks


class TestGrubTemplates(BaseTemplate.BaseTemplateCases):
    """
    Test rendering of jinja2 templates

    When adding or modifying a jinja2 template, add or update the test here.
    Use realistic data - complete exports of the device dictionary preferably.
    Set debug to True to see the content of the rendered templates
    Set system to True to use the system templates - note that this requires
    that the templates in question are in sync with the branch upon which the
    test is run. Therefore, if the templates should be the same, this can be
    used to check that the templates are correct. If there are problems, check
    for a template with a .dpkg-dist extension. Check the diff between the
    checkout and the system file matches the difference between the system file
    and the dpkg-dist version. If the diffs match, copy the dpkg-dist onto the
    system file.
    """

    def test_mustang_pxe_grub_efi_template(self):  # pylint: disable=invalid-name
        data = """{% extends 'mustang-grub-efi.jinja2' %}
{% set hard_reset_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command reboot --port 05' %}
{% set power_off_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command off --port 05' %}
{% set power_on_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command on --port 05' %}
{% set connection_command = 'telnet localhost 7012' %}"""
        self.assertTrue(self.validate_data("staging-mustang-01", data))
        template_dict = prepare_jinja_template("staging-mustang-01", data, raw=False)
        self.assertIn("uefi-menu", template_dict["actions"]["boot"]["methods"])
        self.assertIn(
            "pxe-grub", template_dict["actions"]["boot"]["methods"]["uefi-menu"]
        )
        self.assertNotIn(
            "grub", template_dict["actions"]["boot"]["methods"]["uefi-menu"]
        )
        # label class regex is mangled by jinja/yaml processing
        self.assertNotIn(
            "label_class",
            template_dict["actions"]["boot"]["methods"]["uefi-menu"]["parameters"],
        )
        self.assertIn("grub-efi", template_dict["actions"]["boot"]["methods"])
        self.assertIn(
            "menu_options", template_dict["actions"]["boot"]["methods"]["grub-efi"]
        )
        self.assertEqual(
            template_dict["actions"]["boot"]["methods"]["grub-efi"]["menu_options"],
            "pxe-grub",
        )
        self.assertIn(
            "ramdisk", template_dict["actions"]["boot"]["methods"]["grub-efi"]
        )
        self.assertIn(
            "commands",
            template_dict["actions"]["boot"]["methods"]["grub-efi"]["ramdisk"],
        )
        self.assertIn("nfs", template_dict["actions"]["boot"]["methods"]["grub-efi"])
        self.assertIn(
            "commands", template_dict["actions"]["boot"]["methods"]["grub-efi"]["nfs"]
        )
        nfs_commands = template_dict["actions"]["boot"]["methods"]["grub-efi"]["nfs"][
            "commands"
        ]
        self.assertNotIn("insmod efinet", nfs_commands)
        self.assertNotIn("net_bootp", nfs_commands)

    def test_mustang_grub_efi_template(self):
        data = """{% extends 'mustang-grub-efi.jinja2' %}
{% set grub_efi_method = 'grub' %}
{% set hard_reset_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command reboot --port 05' %}
{% set power_off_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command off --port 05' %}
{% set power_on_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command on --port 05' %}
{% set connection_command = 'telnet localhost 7012' %}"""
        self.assertTrue(self.validate_data("staging-mustang-01", data))
        template_dict = prepare_jinja_template("staging-mustang-01", data, raw=False)
        self.assertIn("uefi-menu", template_dict["actions"]["boot"]["methods"])
        self.assertNotIn(
            "pxe-grub", template_dict["actions"]["boot"]["methods"]["uefi-menu"]
        )
        self.assertIn("grub", template_dict["actions"]["boot"]["methods"]["uefi-menu"])
        self.assertEqual(
            template_dict["actions"]["boot"]["methods"]["grub-efi"]["menu_options"],
            "grub",
        )
        self.assertIn(
            "ramdisk", template_dict["actions"]["boot"]["methods"]["grub-efi"]
        )
        self.assertIn(
            "commands",
            template_dict["actions"]["boot"]["methods"]["grub-efi"]["ramdisk"],
        )
        self.assertIn("nfs", template_dict["actions"]["boot"]["methods"]["grub-efi"])
        self.assertIn(
            "commands", template_dict["actions"]["boot"]["methods"]["grub-efi"]["nfs"]
        )
        nfs_commands = template_dict["actions"]["boot"]["methods"]["grub-efi"]["nfs"][
            "commands"
        ]
        self.assertIn("insmod efinet", nfs_commands)
        self.assertIn("net_bootp", nfs_commands)

    def test_mustang_secondary_media(self):
        data = """{% extends 'mustang-grub-efi.jinja2' %}
{% set sata_label = 'ST500DM002' %}
{% set sata_uuid = 'ata-ST500DM002-1BD142_S2AKYFSN' %}
{% set grub_efi_method = 'pxe-grub' %}
{% set hard_reset_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command reboot --port 05' %}
{% set power_off_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command off --port 05' %}
{% set power_on_command = '/usr/bin/pduclient --daemon services --hostname pdu09 --command on --port 05' %}
{% set connection_command = 'telnet localhost 7012' %}"""
        self.assertTrue(self.validate_data("staging-mustang-01", data))
        template_dict = prepare_jinja_template("staging-mustang-01", data, raw=False)
        parameters = {
            "parameters": {
                "media": {
                    "sata": {
                        "ST500DM002": {
                            "boot_part": 1,
                            "device_id": 0,
                            "grub_interface": "hd0",
                            "uboot_interface": "scsi",
                            "uuid": "ata-ST500DM002-1BD142_S2AKYFSN",
                        },
                        "UUID-required": True,
                    }
                }
            }
        }
        self.assertTrue(template_dict["parameters"] == parameters["parameters"])
        self.assertIn("sata", template_dict["actions"]["boot"]["methods"]["grub-efi"])
        commands = {
            "commands": [
                "insmod gzio",
                "linux (hd0,gpt1)/{KERNEL} console=ttyS0,115200n8 debug root=/dev/sda2 rw ip=:::::eth0:dhcp",
                "initrd (hd0,gpt1/{RAMDISK}",
                "boot",
            ]
        }
        self.assertEqual(
            commands, template_dict["actions"]["boot"]["methods"]["grub-efi"]["sata"]
        )

    def test_overdrive_template(self):
        data = """{% extends 'overdrive.jinja2' %}
{% set connection_command = 'telnet serial4 7001' %}
{% set hard_reset_command = '/usr/local/lab-scripts/snmp_pdu_control --hostname pdu18 --command reboot --port 10 --delay 10' %}
{% set power_off_command = '/usr/local/lab-scripts/snmp_pdu_control --hostname pdu18 --command off --port 10 --delay 10' %}
{% set power_on_command = '/usr/local/lab-scripts/snmp_pdu_control --hostname pdu18 --command on --port 10 --delay 10' %}
{% set map = {'iface0': {'lngswitch03': 13}, 'iface1': {'lngswitch03': 1}, 'iface2': {'lngswitch02': 9}, 'iface3': {'lngswitch02': 10}} %}
{% set tags = {'iface0': [], 'iface1': ['RJ45', '1G', '10G'], 'iface2': ['SFP+', '1G', '10G'], 'iface3': ['SFP+', '1G', '10G']} %}
{% set mac_addr = {'iface0': '00:00:1a:1b:8b:f6', 'iface1': '00:00:1a:1b:8b:f7', 'iface2': '00:11:0a:68:94:30', 'iface3': '00:11:0a:68:94:31'} %}
{% set interfaces = ['iface0', 'iface1', 'iface2', 'iface3'] %}
{% set sysfs = {'iface0': '/sys/devices/platform/AMDI8001:00/net/',
'iface1': '/sys/devices/platform/AMDI8001:01/net/',
'iface2': '/sys/devices/pci0000:00/0000:00:02.1/0000:01:00.0/net/',
'iface3': '/sys/devices/pci0000:00/0000:00:02.1/0000:01:00.1/net/'} %}
{% set boot_character_delay = 100 %}"""
        self.assertTrue(self.validate_data("staging-overdrive-01", data))
        template_dict = prepare_jinja_template("staging-overdrive-01", data, raw=False)
        self.assertIsNotNone(template_dict)
        self.assertIn("parameters", template_dict)
        self.assertIn("interfaces", template_dict["parameters"])
        self.assertIn("actions", template_dict)
        self.assertIn("character_delays", template_dict)
        self.assertIn("boot", template_dict["character_delays"])
        self.assertEqual(100, template_dict["character_delays"]["boot"])
        self.assertIn("iface2", template_dict["parameters"]["interfaces"])
        self.assertIn("iface1", template_dict["parameters"]["interfaces"])
        self.assertIn("iface0", template_dict["parameters"]["interfaces"])
        self.assertIn("sysfs", template_dict["parameters"]["interfaces"]["iface2"])
        self.assertEqual(
            [
                check
                for check in template_dict["actions"]["boot"]["methods"]["grub"]["nfs"][
                    "commands"
                ]
                if "nfsroot" in check
            ][0].count("nfsroot"),
            1,
        )
        self.assertIn(
            " rw",
            [
                check
                for check in template_dict["actions"]["boot"]["methods"]["grub"]["nfs"][
                    "commands"
                ]
                if "nfsroot" in check
            ][0],
        )

    def test_synquacer_acpi_template(self):
        template_dict = self.render_device_dictionary_file(
            "synquacer-acpi-01.jinja2", raw=False
        )
        self.assertIsNotNone(template_dict["actions"]["boot"]["methods"]["grub"])
        self.assertIn("ramdisk", template_dict["actions"]["boot"]["methods"]["grub"])
        self.assertIn(
            "commands", template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]
        )
        self.assertIn(
            "net_add_addr lava efinet0 192.168.25.43",
            template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]["commands"],
        )
        self.assertIn(
            "linux (tftp,{SERVER_IP})/{KERNEL}  ip=192.168.25.43::192.168.25.1:255.255.255.0:::off:192.168.25.1: ",
            template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]["commands"],
        )

    def test_synquacer_dtb_template(self):
        template_dict = self.render_device_dictionary_file(
            "synquacer-dtb-01.jinja2", raw=False
        )
        self.assertIsNotNone(template_dict["actions"]["boot"]["methods"]["grub"])
        self.assertIn("ramdisk", template_dict["actions"]["boot"]["methods"]["grub"])
        self.assertIn(
            "commands", template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]
        )
        self.assertIn(
            "net_add_addr lava efinet0 192.168.25.42",
            template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]["commands"],
        )
        self.assertIn(
            "linux (tftp,{SERVER_IP})/{KERNEL}  ip=192.168.25.42::192.168.25.1:255.255.255.0:::off:192.168.25.1: ",
            template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]["commands"],
        )

    def test_moonshot_m400_template(self):
        template_dict = self.render_device_dictionary_file(
            "moonshot-m400-17.jinja2", raw=False
        )
        self.assertIsNotNone(template_dict["actions"]["boot"]["methods"]["grub"])
        self.assertIn("ramdisk", template_dict["actions"]["boot"]["methods"]["grub"])
        self.assertIn(
            "commands", template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]
        )
        self.assertIn(
            "linux (tftp,{SERVER_IP})/{KERNEL} console=ttyS0,9600 ip=dhcp ",
            template_dict["actions"]["boot"]["methods"]["grub"]["ramdisk"]["commands"],
        )

    def test_minnowboard_turbot_template(self):
        template_dict = self.render_device_dictionary_file(
            "minnowboard-turbot-E3826-01.jinja2", raw=False
        )
        grub = template_dict["actions"]["boot"]["methods"]["grub"]
        self.assertIsNotNone(grub)

        self.assertIn("ramdisk", grub)
        self.assertIn("commands", grub["ramdisk"])
        ramdisk_commands = grub["ramdisk"]["commands"]
        ramdisk_ref_commands = [
            "set net_default_server={SERVER_IP}",
            "linux (tftp)/{KERNEL} console=tty0 console=ttyS0,115200 root=/dev/ram0 ip=:::::eth0:dhcp",
            "initrd (tftp)/{RAMDISK}",
            "boot",
        ]
        self.assertEqual(ramdisk_commands, ramdisk_ref_commands)

        self.assertIn("nfs", grub)
        self.assertIn("commands", grub["nfs"])
        nfs_commands = grub["nfs"]["commands"]
        nfs_ref_commands = [
            "set net_default_server={SERVER_IP}",
            "linux (tftp)/{KERNEL} console=tty0 console=ttyS0,115200 root=/dev/nfs rw nfsroot={NFS_SERVER_IP}:{NFSROOTFS},tcp,hard,intr ip=dhcp",
            "initrd (tftp)/{RAMDISK}",
            "boot",
        ]
        self.assertEqual(nfs_commands, nfs_ref_commands)

    def test_qdf2400_template(self):
        template_dict = self.render_device_dictionary_file(
            "qcom-qdf2400-01.jinja2", raw=False
        )
        grub = template_dict["actions"]["boot"]["methods"]["grub"]
        self.assertIsNotNone(grub)
        nfs_commands = template_dict["actions"]["boot"]["methods"]["grub"]["nfs"][
            "commands"
        ]
        self.assertIn("insmod efinet", nfs_commands)
        self.assertNotIn("net_bootp", nfs_commands)



# -*- coding: utf-8 -*-
from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("lava_scheduler_app", "0005_auto_devicedictionarytable_pipelinestore")
    ]

    operations = [
        migrations.CreateModel(
            name="Architecture",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="e.g. ARMv7",
                        max_length=100,
                        serialize=False,
                        verbose_name="Architecture version",
                        primary_key=True,
                    ),
                )
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="BitWidth",
            fields=[
                (
                    "width",
                    models.PositiveSmallIntegerField(
                        help_text="integer: e.g. 32 or 64",
                        serialize=False,
                        verbose_name="Processor bit width",
                        primary_key=True,
                    ),
                )
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="Core",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="Name of a specific CPU core, e.g. Cortex-A9",
                        max_length=100,
                        serialize=False,
                        verbose_name="CPU core",
                        primary_key=True,
                    ),
                )
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="ProcessorFamily",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="e.g. OMAP4, Exynos",
                        max_length=100,
                        serialize=False,
                        verbose_name="Processor Family",
                        primary_key=True,
                    ),
                )
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name="devicetype",
            name="architecture",
            field=models.ForeignKey(
                related_name="device_types",
                blank=True,
                to="lava_scheduler_app.Architecture",
                null=True,
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="devicetype",
            name="bits",
            field=models.ForeignKey(
                related_name="device_types",
                blank=True,
                to="lava_scheduler_app.BitWidth",
                null=True,
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="devicetype",
            name="core_count",
            field=models.PositiveSmallIntegerField(
                help_text="Must be an equal number of each type(s) of core(s).",
                null=True,
                verbose_name="Total number of cores",
                blank=True,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="devicetype",
            name="cores",
            field=models.ManyToManyField(
                related_name="device_types",
                null=True,
                to="lava_scheduler_app.Core",
                blank=True,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="devicetype",
            name="cpu_model",
            field=models.CharField(
                help_text="e.g. a list of CPU model descriptive strings: OMAP4430 / OMAP4460",
                max_length=100,
                null=True,
                verbose_name="CPU model",
                blank=True,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="devicetype",
            name="processor",
            field=models.ForeignKey(
                related_name="device_types",
                blank=True,
                to="lava_scheduler_app.ProcessorFamily",
                null=True,
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2015-2018 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#         Stevan Radakovic <stevan.radakovic@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

import django_tables2 as tables

from django.contrib.contenttypes.models import ContentType
from django.utils.safestring import mark_safe
from django.utils.html import escape

from lava.utils.lavatable import LavaTable
from lava_scheduler_app.tables import RestrictedIDLinkColumn
from lava_results_app.models import TestCase, BugLink, TestSuite


def results_pklink(record):
    job_id = record.pk
    complete = (
        '<a class="btn btn-xs btn-success pull-right" title="test job results" href="%s">'
        % record.results_link
    )
    button = '<span class="glyphicon glyphicon-signal"></span></a>'
    return mark_safe(
        '<a href="%s" title="test job summary">%s</a>&nbsp;%s%s'
        % (record.get_absolute_url(), escape(job_id), complete, button)
    )


class IndexResultsColumn(RestrictedIDLinkColumn):
    def render(self, record, table=None):
        user = table.context.get("request").user
        if record.job.can_view(user):
            return results_pklink(record.job)
        else:
            return record.job.pk


class ResultsTable(LavaTable):
    """
    List of LAVA TestSuite results
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.length = 25

    def _check_job(self, record, table=None):  # pylint: disable=no-self-use
        """
        Slightly different purpose to RestrictedIDLinkColumn.render
        """
        user = table.context.get("request").user
        return bool(record.job.can_view(user))

    def render_submitter(self, record, table=None):
        if not self._check_job(record, table):
            return "Unavailable"
        return record.job.submitter

    def render_passes(self, record, table=None):
        if not self._check_job(record, table):
            return ""
        return record.testcase_count("pass")

    def render_fails(self, record, table=None):
        if not self._check_job(record, table):
            return ""
        return record.testcase_count("fail")

    def render_total(self, record, table=None):
        if not self._check_job(record, table):
            return ""
        return TestCase.objects.filter(suite__job=record.job, suite=record).count()

    def render_logged(self, record, table=None):
        if not self._check_job(record, table):
            return ""
        try:
            return TestCase.objects.filter(suite__job=record.job, suite=record)[
                0
            ].logged
        except TestCase.DoesNotExist:
            return record.job.start_time

    def render_buglinks(self, record, table=None):

        suite_links_count = BugLink.objects.filter(
            content_type=ContentType.objects.get_for_model(TestSuite),
            object_id=record.id,
        ).count()
        case_links_count = BugLink.objects.filter(
            content_type=ContentType.objects.get_for_model(TestCase),
            object_id__in=TestCase.objects.filter(suite=record),
        ).count()

        user = table.context.get("request").user
        if not user.is_anonymous:
            return mark_safe(
                '<a href="#" class="buglink" id="buglink_%s" data-content-type="%s">[%s]</a> (%s)'
                % (
                    record.id,
                    ContentType.objects.get_for_model(TestSuite).id,
                    suite_links_count,
                    case_links_count,
                )
            )
        else:
            return mark_safe("[%s] (%s)" % (suite_links_count, case_links_count))

    job_id = tables.Column(verbose_name="Job ID")
    actions = tables.TemplateColumn(
        template_name="lava_results_app/results_actions_field.html"
    )
    actions.orderable = False
    submitter = tables.Column(accessor="job.submitter")
    name = tables.Column(verbose_name="Test Suite")
    passes = tables.Column(accessor="job", verbose_name="Passes")
    fails = tables.Column(accessor="job", verbose_name="Fails")
    total = tables.Column(accessor="job", verbose_name="Totals")
    logged = tables.Column(accessor="job", verbose_name="Logged")
    buglinks = tables.Column(accessor="job", verbose_name="Bug Links")
    buglinks.orderable = False

    class Meta(LavaTable.Meta):  # pylint: disable=no-init,too-few-public-methods
        searches = {"name": "contains"}
        sequence = {"job_id", "actions"}


class ResultsIndexTable(ResultsTable):

    job_id = tables.Column(verbose_name="Job ID")
    submitter = tables.Column(accessor="job.submitter")
    name = tables.Column(verbose_name="Test Suite")
    passes = tables.Column(accessor="job", verbose_name="Passes")
    fails = tables.Column(accessor="job", verbose_name="Fails")
    total = tables.Column(accessor="job", verbose_name="Totals")
    logged = tables.Column(accessor="job", verbose_name="Logged")

    class Meta(LavaTable.Meta):  # pylint: disable=no-init,too-few-public-methods
        searches = {"name": "contains"}


class TestJobResultsTable(ResultsTable):

    job_id = tables.Column(verbose_name="Job ID")
    actions = tables.TemplateColumn(
        template_name="lava_results_app/suite_actions_field.html"
    )
    actions.orderable = False
    submitter = tables.Column(accessor="job.submitter")
    name = tables.Column(verbose_name="Test Suite")
    passes = tables.Column(accessor="job", verbose_name="Passes")
    fails = tables.Column(accessor="job", verbose_name="Fails")
    total = tables.Column(accessor="job", verbose_name="Totals")
    logged = tables.Column(accessor="job", verbose_name="Logged")

    class Meta(LavaTable.Meta):  # pylint: disable=no-init,too-few-public-methods
        searches = {"name": "contains"}


class SuiteTable(LavaTable):
    """
    Details of the test sets or test cases in a test suite
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.length = 25

    name = tables.Column()
    test_set = tables.Column(verbose_name="Test Set")
    result = tables.Column()
    measurement = tables.Column()
    units = tables.Column()
    logged = tables.DateTimeColumn()
    buglinks = tables.Column(accessor="suite", verbose_name="Bug Links")
    buglinks.orderable = False

    def render_name(self, record):  # pylint: disable=no-self-use
        return mark_safe(
            '<a href="%s">%s</a>' % (record.get_absolute_url(), record.name)
        )

    def render_result(self, record):  # pylint: disable=no-self-use
        code = record.result_code
        if code == "pass":
            icon = "ok"
        elif code == "fail":
            icon = "remove"
        else:
            icon = "minus"
        return mark_safe(
            '<a href="%s"><span class="glyphicon glyphicon-%s"></span> %s</a>'
            % (record.get_absolute_url(), icon, code)
        )

    def render_test_set(self, record):  # pylint: disable=no-self-use
        return mark_safe(
            '<a href="%s">%s</a>'
            % (record.test_set.get_absolute_url(), record.test_set.name)
        )

    def render_buglinks(self, record, table=None):  # pylint: disable=no-self-use
        case_links_count = BugLink.objects.filter(
            content_type=ContentType.objects.get_for_model(TestCase),
            object_id=record.id,
        ).count()

        user = table.context.get("request").user
        if not user.is_anonymous:
            return mark_safe(
                '<a href="#" class="buglink" id="buglink_%s" data-content-type="%s">[%s]</a>'
                % (
                    record.id,
                    ContentType.objects.get_for_model(TestCase).id,
                    case_links_count,
                )
            )
        else:
            return mark_safe("[%s]" % (case_links_count))

    class Meta(LavaTable.Meta):  # pylint: disable=no-init,too-few-public-methods
        searches = {"name": "contains"}



# Copyright (C) 2014 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

import atexit
import os
import shutil
import tarfile
import tempfile
import guestfs
import glob
import logging
import magic
import errno
from configobj import ConfigObj

from lava_common.exceptions import InfrastructureError, JobError, LAVABug
from lava_common.constants import LXC_PATH, LAVA_LXC_HOME
from lava_dispatcher.utils.compression import decompress_file
from lava_dispatcher.utils.decorator import replace_exception


def rmtree(directory):
    """
    Wrapper around shutil.rmtree to remove a directory tree while ignoring most
    errors.
    If called on a symbolic link, this function will raise a LAVABug.
    """
    # TODO: consider how to handle problems if the directory has already been removed -
    # coding bugs may trigger this Runtime exception - implement before moving to production.
    try:
        shutil.rmtree(directory)
    except OSError as exc:
        raise LAVABug("Error when trying to remove '%s': %s" % (directory, exc))


def mkdtemp(autoremove=True, basedir="/tmp"):  # nosec - internal use.
    """
    returns a temporary directory that's deleted when the process exits

    """
    tmpdir = tempfile.mkdtemp(dir=basedir)
    os.chmod(tmpdir, 0o755)  # nosec - internal use.
    if autoremove:
        atexit.register(rmtree, tmpdir)
    return tmpdir


def check_ssh_identity_file(params):  # pylint: disable=too-many-return-statements
    """
    Return a tuple based on if an identity file can be determine in the params.
    If the first value returned is not None, an error occurred.
    If the first value is None, the second value contains the path to the identity_file
    """
    if not params or not isinstance(params, dict):
        return "Invalid parameters", None
    if "ssh" not in params:
        return "Empty ssh parameter list in device configuration %s" % params, None
    if os.path.isabs(params["ssh"]["identity_file"]):
        identity_file = params["ssh"]["identity_file"]
    else:
        identity_file = os.path.realpath(
            os.path.join(
                __file__,
                "../"
                * 2,  # up two directories from this file - not a constant, a result of the layout.
                params["ssh"]["identity_file"],
            )
        )
    if not os.path.exists(identity_file):
        return "Cannot find SSH private key %s" % identity_file, None
    if not os.path.exists("%s.pub" % identity_file):
        return "Cannot find SSH public key %s.pub" % identity_file, None
    return None, identity_file


def tftpd_dir():
    """
    read in 'TFTP_DIRECTORY' from /etc/default/tftpd-hpa
    Any file to be offered using tftp must use this directory or a
    subdirectory of it. Default installation value: /srv/tftp/
    :return: real path to the TFTP directory or raises InfrastructureError
    """
    var_name = "TFTP_DIRECTORY"
    if os.path.exists("/etc/default/tftpd-hpa"):
        config = ConfigObj("/etc/default/tftpd-hpa")
        value = config.get(var_name)
        return os.path.realpath(value)
    raise InfrastructureError("Unable to identify tftpd directory")


def write_bootscript(commands, filename):
    with open(filename, "w") as bootscript:
        bootscript.write("#!ipxe\n\n")
        for line in commands:
            bootscript.write(line + "\n")
        bootscript.close()


def _launch_guestfs(guest):
    # Launch guestfs and raise an InfrastructureError if needed
    try:
        guest.launch()
    except RuntimeError as exc:
        logger = logging.getLogger("dispatcher")
        logger.exception(str(exc))
        raise InfrastructureError("Unable to start libguestfs")


@replace_exception(RuntimeError, JobError)
def prepare_guestfs(output, overlay, size):
    """
    Applies the overlay, offset by one directory.
    This allows the booted device to mount at the
    original lava directory and retain the same path
    as if the overlay was unpacked directly into the
    image.
    :param output: filename of the temporary device
    :param overlay: tarball of the lava test shell overlay.
    :param size: size of the filesystem in Mb
    :return blkid of the guest device
    """
    guest = guestfs.GuestFS(python_return_dict=True)
    guest.disk_create(output, "qcow2", size * 1024 * 1024)
    guest.add_drive_opts(output, format="qcow2", readonly=False)
    _launch_guestfs(guest)
    devices = guest.list_devices()
    if len(devices) != 1:
        raise InfrastructureError("Unable to prepare guestfs")
    guest_device = devices[0]
    guest.mke2fs(guest_device, label="LAVA")
    # extract to a temp location
    tar_output = mkdtemp()
    # Now mount the filesystem so that we can add files.
    guest.mount(guest_device, "/")
    tarball = tarfile.open(overlay)
    tarball.extractall(tar_output)
    guest_dir = mkdtemp()
    guest_tar = os.path.join(guest_dir, "guest.tar")
    root_tar = tarfile.open(guest_tar, "w")
    for topdir in os.listdir(tar_output):
        for dirname in os.listdir(os.path.join(tar_output, topdir)):
            root_tar.add(os.path.join(tar_output, topdir, dirname), arcname=dirname)
    root_tar.close()
    guest.tar_in(guest_tar, "/")
    os.unlink(guest_tar)
    guest.umount(guest_device)
    return guest.blkid(guest_device)["UUID"]


@replace_exception(RuntimeError, JobError)
def prepare_install_base(output, size):
    """
    Create an empty image of the specified size (in bytes),
    ready for an installer to partition, create filesystem(s)
    and install files.
    """
    guest = guestfs.GuestFS(python_return_dict=True)
    guest.disk_create(output, "raw", size)
    guest.add_drive_opts(output, format="raw", readonly=False)
    _launch_guestfs(guest)
    devices = guest.list_devices()
    if len(devices) != 1:
        raise InfrastructureError("Unable to prepare guestfs")
    guest.shutdown()


@replace_exception(RuntimeError, JobError)
def copy_out_files(image, filenames, destination):
    """
    Copies a list of files out of the image to the specified
    destination which must exist. Launching the guestfs is
    expensive, so copy out all files in one operation. The
    filenames list must contain unique filenames even if the
    source files exist in separate directories.
    """
    if not isinstance(filenames, list):
        raise LAVABug("filenames must be a list")
    guest = guestfs.GuestFS(python_return_dict=True)
    guest.add_drive_ro(image)
    _launch_guestfs(guest)
    devices = guest.list_devices()
    if len(devices) != 1:
        raise InfrastructureError("Unable to prepare guestfs")
    guest.mount_ro(devices[0], "/")
    for filename in filenames:
        guest.copy_out(filename, destination)
    guest.shutdown()


@replace_exception(RuntimeError, JobError)
def copy_in_overlay(image, root_partition, overlay):
    """
    Mounts test image partition as specified by the test
    writer and extracts overlay at the root, if root_partition
    is None the image is handled as a filesystem instead of
    partitioned image.
    """
    guest = guestfs.GuestFS(python_return_dict=True)
    guest.add_drive(image)
    _launch_guestfs(guest)

    if root_partition:
        partitions = guest.list_partitions()
        if not partitions:
            raise InfrastructureError("Unable to prepare guestfs")
        guest_partition = partitions[root_partition]
        guest.mount(guest_partition, "/")
    else:
        devices = guest.list_devices()
        if not devices:
            raise InfrastructureError("Unable to prepare guestfs")
        guest.mount(devices[0], "/")

    # FIXME: max message length issues when using tar_in
    # on tar.gz.  Works fine with tar so decompressing
    # overlay first.
    if os.path.exists(overlay[:-3]):
        os.unlink(overlay[:-3])
    decompressed_overlay = decompress_file(overlay, "gz")
    guest.tar_in(decompressed_overlay, "/")

    if root_partition:
        guest.umount(guest_partition)
    else:
        guest.umount(devices[0])


def lxc_path(dispatcher_config):
    """
    Returns LXC_PATH which is a constant, unless a dispatcher specific path is
    configured via lxc_path key in dispatcher_config.
    """
    try:
        return dispatcher_config["lxc_path"]
    except (KeyError, TypeError):
        return LXC_PATH


def lava_lxc_home(lxc_name, dispatcher_config):
    """
    Creates lava_lxc_home if it is unavailable and Returns absolute path of
    LAVA_LXC_HOME as seen from the host machine.

    Takes into account the dispatcher specific path configured via lxc_path
    key in dispatcher_config.
    """
    path = os.path.join(
        lxc_path(dispatcher_config), lxc_name, "rootfs", LAVA_LXC_HOME.lstrip("/")
    )
    # Create lava_lxc_home if it is unavailable
    os.makedirs(path, 0o755, exist_ok=True)
    return path


def copy_to_lxc(lxc_name, src, dispatcher_config):
    """Copies given file in SRC to LAVA_LXC_HOME with the provided LXC_NAME
    and configured lxc_path

    For example,

    SRC such as:
    '/var/lib/lava/dispatcher/tmp/tmpuuI_U0/system.img'

    will get copied to:
    '/var/lib/lxc/lxc-nexus4-test-None/rootfs/lava-lxc/system.img'

    where, '/var/lib/lxc' is the lxc_path and 'lxc-nexus4-test-None' is the
    LXC_NAME

    Returns the destination path within lxc. For example, '/lava-lxc/boot.img'

    Raises JobError if the copy failed.
    """
    filename = os.path.basename(src)
    dst = os.path.join(lava_lxc_home(lxc_name, dispatcher_config), filename)
    logger = logging.getLogger("dispatcher")
    if src == dst:
        logger.debug("Not copying since src: '%s' and dst: '%s' are same", src, dst)
    else:
        logger.debug("Copying %s to %s", filename, lxc_name)
        try:
            shutil.copyfile(src, dst)
        except OSError:
            raise JobError("Unable to copy image: %s" % src)

    return os.path.join(LAVA_LXC_HOME, filename)


def copy_overlay_to_lxc(lxc_name, src, dispatcher_config, namespace):
    """Copies given overlay tar file in SRC to LAVA_LXC_HOME with the provided
    LXC_NAME and configured lxc_path

    For example,

    SRC such as:
    '/var/lib/lava/dispatcher/slave/tmp/523/overlay-1.8.4.tar.gz'

    will get copied to:
    '/var/lib/lxc/db410c-523/rootfs/lava-lxc/overlays/${namespace}/overlay.tar.gz'

    where,
    '/var/lib/lxc' is the lxc_path
    'db410c-523' is the LXC_NAME
    ${namespace} is the given NAMESPACE

    Returns the destination path. For example,
    '/var/lib/lxc/db410c-523/rootfs/lava-lxc/overlays/${namespace}/overlay.tar.gz'

    Raises JobError if the copy failed.
    """
    dst = os.path.join(
        lava_lxc_home(lxc_name, dispatcher_config),
        "overlays",
        namespace,
        "overlay.tar.gz",
    )
    logger = logging.getLogger("dispatcher")
    logger.debug("Copying %s to %s", os.path.basename(src), dst)
    try:
        shutil.copy(src, dst)
    except OSError as exc:
        # ENOENT(2): No such file or directory
        if exc.errno != errno.ENOENT:
            raise JobError("Unable to copy image: %s" % src)
        # try creating parent directories
        os.makedirs(os.path.dirname(dst), 0o755)
        shutil.copy(src, dst)
    return dst


@replace_exception(RuntimeError, JobError)
def copy_overlay_to_sparse_fs(image, overlay):
    """copy_overlay_to_sparse_fs

    Only copies the overlay to an image
    which has already been converted from sparse.
    """
    logger = logging.getLogger("dispatcher")
    guest = guestfs.GuestFS(python_return_dict=True)
    guest.add_drive(image)
    _launch_guestfs(guest)
    devices = guest.list_devices()
    if not devices:
        raise InfrastructureError("Unable to prepare guestfs")
    guest.mount(devices[0], "/")
    # FIXME: max message length issues when using tar_in
    # on tar.gz.  Works fine with tar so decompressing
    # overlay first.
    if os.path.exists(overlay[:-3]):
        os.unlink(overlay[:-3])
    decompressed_overlay = decompress_file(overlay, "gz")
    guest.tar_in(decompressed_overlay, "/")
    # Check if we have space left on the mounted image.
    output = guest.df()
    logger.debug(output)
    _, _, _, available, percent, _ = output.split("\n")[1].split()
    guest.umount(devices[0])
    if int(available) is 0 or percent == "100%":
        raise JobError("No space in image after applying overlay: %s" % image)


def copy_directory_contents(root_dir, dst_dir):
    """
    Copies the contents of the root directory to the destination directory
    but excludes the root directory's top level folder
    """
    files_to_copy = glob.glob(os.path.join(root_dir, "*"))
    logger = logging.getLogger("dispatcher")
    for fname in files_to_copy:
        logger.debug(
            "copying %s to %s", fname, os.path.join(dst_dir, os.path.basename(fname))
        )
        if os.path.isdir(fname):
            shutil.copytree(fname, os.path.join(dst_dir, os.path.basename(fname)))
        else:
            shutil.copy(fname, dst_dir)


def remove_directory_contents(root_dir):
    """
    Removes the contents of the root directory but not the root itself
    """
    files_to_remove = glob.glob(os.path.join(root_dir, "*"))
    logger = logging.getLogger("dispatcher")
    for fname in files_to_remove:
        logger.debug("removing %s", fname)
        if os.path.isdir(fname):
            shutil.rmtree(fname)
        else:
            os.remove(fname)


def is_sparse_image(image):
    """
    Returns True if the image is an 'Android sparse image' else False.
    """
    image_magic = magic.open(magic.MAGIC_NONE)  # pylint: disable=no-member
    image_magic.load()
    return bool(image_magic.file(image).split(",")[0] == "Android sparse image")



# Copyright (C) 2018 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from lava_dispatcher.action import Pipeline
from lava_dispatcher.logical import Boot, RetryAction
from lava_dispatcher.actions.boot import (
    BootAction,
    BootloaderCommandsAction,
    BootloaderCommandOverlay,
    BootloaderInterruptAction,
)
from lava_dispatcher.power import ResetDevice
from lava_dispatcher.connections.serial import ConnectDevice


class BootBootloader(Boot):
    compatibility = 4

    def __init__(self, parent, parameters):
        super().__init__(parent)
        self.action = BootBootloaderAction()
        self.action.section = self.action_type
        self.action.job = self.job
        parent.add_action(self.action, parameters)

    @classmethod
    def accepts(cls, device, parameters):
        if parameters["method"] != "bootloader":
            return False, "'method' was not 'bootloader'"
        if "bootloader" not in parameters:
            return False, "'bootloader' is undefined"
        bootloader = parameters["bootloader"]
        if bootloader not in device["actions"]["boot"]["methods"]:
            return (
                False,
                "boot method '%s' not in the device configuration" % bootloader,
            )
        return True, "accepted"


class BootBootloaderAction(BootAction):

    name = "boot-bootloader"
    description = "boot to bootloader"
    summary = "boot bootloader"

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        self.internal_pipeline.add_action(ConnectDevice())
        self.internal_pipeline.add_action(
            BootloaderCommandOverlay(method=parameters["bootloader"])
        )
        self.internal_pipeline.add_action(BootBootloaderRetry())


class BootBootloaderRetry(RetryAction):

    name = "boot-bootloader-retry"
    description = "boot to bootloader with retry"
    summary = "boot bootloader retry"

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        self.internal_pipeline.add_action(ResetDevice())
        self.internal_pipeline.add_action(
            BootloaderInterruptAction(method=parameters["bootloader"])
        )
        self.internal_pipeline.add_action(
            BootloaderCommandsAction(
                expect_final=False, method=parameters["bootloader"]
            )
        )



# -*- coding: utf-8 -*-
# Copyright (C) 2018 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

import contextlib
from django.core.management.base import BaseCommand, CommandError, CommandParser
from lava_scheduler_app.models import Device, Tag


class Command(BaseCommand):
    help = "Manage device tags"

    def add_arguments(self, parser):
        cmd = self

        class SubParser(CommandParser):
            """
            Sub-parsers constructor that mimic Django constructor.
            See http://stackoverflow.com/a/37414551
            """

            def __init__(self, **kwargs):
                super().__init__(cmd, **kwargs)

        sub = parser.add_subparsers(
            dest="sub_command", help="Sub commands", parser_class=SubParser
        )
        sub.required = True

        # "add" sub-command
        add_parser = sub.add_parser(
            "add", help="Add one or more existing device tag(s) to an existing device"
        )
        add_parser.add_argument("hostname", help="Hostname of the device")
        add_parser.add_argument(
            "--tags",
            nargs="*",
            required=False,
            help="List of tags to add to the device",
        )

        # "show" sub-command
        show_parser = sub.add_parser(
            "show", help="Show all device tag(s) for an existing device"
        )
        show_parser.add_argument("hostname", help="Hostname of the device")

        # "list" sub-command
        sub.add_parser("list", help="List all device tag(s)")

        # "remove" sub-command
        remove_parser = sub.add_parser(
            "remove",
            help="Remove one or more existing device tag(s) from an existing device",
        )
        remove_parser.add_argument("hostname", help="Hostname of the device")
        remove_parser.add_argument(
            "--tags",
            nargs="*",
            required=False,
            help="List of tags to remove from the device",
        )

        # "create" sub-command
        create_parser = sub.add_parser("create", help="Create a new device tag")
        create_parser.add_argument("name", help="Name of the new tag")
        create_parser.add_argument("description", help="Description of the new tag")

    def handle(self, *args, **options):
        """ Forward to the right sub-handler """
        if options["sub_command"] == "add":
            self.handle_add(options)
        elif options["sub_command"] == "list":
            self.handle_list(options)
        elif options["sub_command"] == "show":
            self.handle_show(options)
        elif options["sub_command"] == "remove":
            self.handle_remove(options)
        else:
            self.handle_create(options)

    def handle_add(self, options):
        hostname = options["hostname"]
        try:
            device = Device.objects.get(hostname=hostname)
        except Device.DoesNotExist:
            raise CommandError("Device '%s' does NOT exist!" % hostname)

        tags = options["tags"]
        if tags is not None:
            for tag in tags:
                try:
                    tag = Tag.objects.get(name=tag)
                except Tag.DoesNotExist:
                    raise CommandError("Tag '%s' does NOT exist!" % tag)
                device.tags.add(tag)
        device.save()

    def handle_show(self, options):
        hostname = options["hostname"]
        try:
            device = Device.objects.get(hostname=hostname)
        except Device.DoesNotExist:
            raise CommandError("Device '%s' does NOT exist!" % hostname)
        if device.tags.all():
            self.stdout.write("Hostname: %s" % hostname)
        for tag in device.tags.all():
            self.stdout.write(
                "Tag name: '%s' Description: '%s'" % (tag.name, tag.description)
            )

    def handle_remove(self, options):
        hostname = options["hostname"]
        try:
            device = Device.objects.get(hostname=hostname)
        except Device.DoesNotExist:
            raise CommandError("Device '%s' does NOT exist!" % hostname)

        tags = options["tags"]
        if tags is not None:
            for tag in tags:
                try:
                    tag = Tag.objects.get(name=tag)
                except Device.DoesNotExist:
                    raise CommandError("Tag '%s' does NOT exist!" % tag)
                if tag not in device.tags.all():
                    raise CommandError(
                        "Device %s does not have tag %s" % (hostname, tag.name)
                    )
                device.tags.remove(tag)
        device.save()

    def handle_create(self, options):
        name = options["name"]
        description = options["description"]

        with contextlib.suppress(Tag.DoesNotExist):
            Tag.objects.get(name=name)
            raise CommandError("Tag '%s' already exists" % name)
        Tag.objects.create(name=name, description=description)

    def handle_list(self, options):
        for tag in Tag.objects.all():
            self.stdout.write(
                "Name: '%s' Description: '%s'" % (tag.name, tag.description)
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  download-test-suites-api.py
#
#  Copyright 2018 Linaro Limited
#  Author: Stevan Radakovic <stevan.radakovic@linaro.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse
import requests
import yaml


def main():
    parser = argparse.ArgumentParser(description="LAVA test suite download showcase")
    parser.add_argument("--instance", type=str, required=True, help="XMLRPC endpoint")
    parser.add_argument(
        "--username", default=None, type=str, help="Username used for authentication"
    )
    parser.add_argument(
        "--token", default=None, type=str, help="Token used for authentication"
    )
    parser.add_argument("--job-id", type=str, required=True, help="Job ID")
    parser.add_argument(
        "--https", action="store_true", help="Use https instead of http"
    )
    parser.add_argument(
        "testsuites",
        nargs="+",
        help="Specify list of test suite names which should be downloaded",
    )
    args = parser.parse_args()

    protocol = "https" if args.https else "http"

    job_results_url = "%s://%s/results/%s" % (protocol, args.instance, args.job_id)
    summary_response = requests.get(
        "%s/yaml_summary" % (job_results_url),
        params={"user": args.username, "token": args.token},
    )

    for test_suite in yaml.safe_load(summary_response.content):
        if test_suite["name"] in args.testsuites:
            print("Test case list for suite '%s':" % test_suite["name"])
            suite_response = requests.get(
                "%s/%s/yaml" % (job_results_url, test_suite["name"]),
                params={"user": args.username, "token": args.token},
            )

            for test_case in yaml.safe_load(suite_response.content):
                print("- %s" % test_case["name"])


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-08 17:08
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0035_remove_testjob__results_link")]

    operations = [
        migrations.RemoveField(model_name="device", name="is_pipeline"),
        migrations.RemoveField(model_name="testjob", name="is_pipeline"),
    ]



# Copyright (C) 2018 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

import pytest
import subprocess  # nosec - only for mocking
import time

from lava_common.exceptions import InfrastructureError, JobError
from lava_dispatcher.actions.test.interactive import TestInteractiveAction

from lava_dispatcher.tests.test_basic import Factory, StdoutTestCase

# This will be monkey patched
import lava_dispatcher.actions.deploy.docker


class InteractiveFactory(Factory):
    def create_interactive_job(self, device, filename):
        return self.create_job(device, filename)


def test_pipeline():
    factory = InteractiveFactory()
    job = factory.create_interactive_job(
        "b2260-01.jinja2", "sample_jobs/b2260-interactive.yaml"
    )
    job.validate()
    description_ref = StdoutTestCase.pipeline_reference(
        "b2260-interactive.yaml", job=job
    )
    assert description_ref == job.pipeline.describe(False)  # nosec


def test_stages(monkeypatch):
    monkeypatch.setattr(subprocess, "check_output", lambda cmd: b"")
    monkeypatch.setattr(
        lava_dispatcher.actions.deploy.docker, "which", lambda a: "/usr/bin/docker"
    )
    factory = InteractiveFactory()
    job = factory.create_interactive_job(
        "docker-01.jinja2", "sample_jobs/docker-interactive.yaml"
    )
    job.validate()
    description_ref = StdoutTestCase.pipeline_reference(
        "docker-interactive.yaml", job=job
    )
    assert description_ref == job.pipeline.describe(False)  # nosec
    assert (  # nosec  - assert is part of the test process.
        job.pipeline.actions[3].internal_pipeline.actions[0].parameters["stage"] == 0
    )


def test_raise_exception():
    action = TestInteractiveAction()
    action.validate()

    with pytest.raises(InfrastructureError) as exc:
        action.raise_exception("InfrastructureError", "hello")
    assert exc.match("hello")  # nosec - assert is part of the test process.

    with pytest.raises(JobError) as exc:
        action.raise_exception("JobError", "A strange error")
    assert exc.match("A strange error")  # nosec - assert is part of the test process.

    with pytest.raises(JobError) as exc:
        action.raise_exception("bug", "A strange error")
    assert exc.match(  # nosec - assert is ok
        "Unknow exception 'bug' with 'A strange error'"
    )


class Connection:
    def __init__(self, data):
        self.data = data

    def sendline(self, line, delay):
        assert self.data.pop(0) == ("sendline", line)  # nosec - assert is ok

    def expect(self, expect, timeout=1):
        data = self.data.pop(0)
        assert (data[0], data[1]) == ("expect", expect)  # nosec - assert is ok
        return data[2]


class Logger:
    def __init__(self, data):
        self.data = data

    def debug(self, line, *args):
        assert self.data.pop(0) == ("debug", line % args)  # nosec - assert is ok

    def error(self, line, *args):
        assert self.data.pop(0) == ("error", line % args)  # nosec - assert is ok

    def info(self, line, *args):
        assert self.data.pop(0) == ("info", line % args)  # nosec - assert is ok

    def results(self, res):
        assert self.data.pop(0) == ("results", res)  # nosec - assert is ok


class Timing:
    def __init__(self):
        self.clock = 0

    def __call__(self):
        self.clock += 1
        return self.clock


def test_run_script(monkeypatch):
    monkeypatch.setattr(time, "time", Timing())
    action = TestInteractiveAction()
    action.parameters = {"stage": 0}
    action.logger = Logger(
        [
            ("info", "Sending nothing, waiting"),
            ("debug", "Waiting for '=> ', '/ # '"),
            ("debug", "Matched a prompt: '=> '"),
            (
                "results",
                {
                    "definition": "0_setup",
                    "case": "wait prompt",
                    "result": "pass",
                    "duration": "1.00",
                },
            ),
            ("info", "Sending 'help'"),
            ("debug", "Waiting for '=> ', '/ # '"),
            ("debug", "Matched a prompt: '=> '"),
            (
                "results",
                {
                    "definition": "0_setup",
                    "case": "network.help",
                    "result": "pass",
                    "duration": "1.00",
                },
            ),
            ("info", "Sending 'dhcp'"),
            ("debug", "Waiting for '=> ', '/ # ', 'DHCP client bound to address'"),
            ("info", "Matched a success: 'DHCP client bound to address'"),
            (
                "results",
                {
                    "definition": "0_setup",
                    "case": "network.dhcp",
                    "result": "pass",
                    "duration": "1.00",
                },
            ),
            ("info", "Sending 'host lavasoftware.org'"),
            (
                "debug",
                "Waiting for '=> ', '/ # ', 'Host lavasoftware.org not found: 3(NXDOMAIN)', 'TIMEOUT'",
            ),
            (
                "error",
                "Matched a failure: 'Host lavasoftware.org not found: 3(NXDOMAIN)'",
            ),
            (
                "results",
                {
                    "definition": "0_setup",
                    "case": "network.host",
                    "result": "fail",
                    "duration": "1.00",
                },
            ),
            ("info", "Sending 'dig lavasoftware.org'"),
            ("debug", "Waiting for '=> ', '/ # ', '192.168.0.1'"),
            ("error", "Matched a prompt (was expecting a success): '=> '"),
            (
                "results",
                {
                    "definition": "0_setup",
                    "case": "network.dig",
                    "result": "fail",
                    "duration": "1.00",
                },
            ),
            ("info", "Sending 'ping -c 1 lavasoftware.org'"),
            ("debug", "Waiting for '=> ', '/ # ', 'Name or service not known'"),
            ("error", "Matched a failure: 'Name or service not known'"),
            (
                "results",
                {
                    "definition": "0_setup",
                    "case": "network.ping",
                    "result": "fail",
                    "duration": "1.00",
                },
            ),
        ]
    )
    action.validate()

    conn_data = [
        ("expect", ["=> ", "/ # "], 0),
        ("sendline", "help"),
        ("expect", ["=> ", "/ # "], 0),
        ("sendline", "dhcp"),
        ("expect", ["=> ", "/ # ", "DHCP client bound to address"], 2),
        ("expect", ["=> ", "/ # "], 0),
        ("sendline", "host lavasoftware.org"),
        (
            "expect",
            ["=> ", "/ # ", "Host lavasoftware.org not found: 3(NXDOMAIN)", "TIMEOUT"],
            2,
        ),
        ("sendline", "dig lavasoftware.org"),
        ("expect", ["=> ", "/ # ", "192.168.0.1"], 0),
        ("sendline", "ping -c 1 lavasoftware.org"),
        ("expect", ["=> ", "/ # ", "Name or service not known"], 2),
    ]
    script = {
        "prompts": ["=> ", "/ # "],
        "name": "setup",
        "script": [
            {"command": None, "name": "wait prompt"},
            {"command": "help", "name": "network.help"},
            {
                "command": "dhcp",
                "name": "network.dhcp",
                "successes": [{"message": "DHCP client bound to address"}],
            },
            {
                "command": "host {HOST}",
                "name": "network.host",
                "failures": [
                    {"message": "Host lavasoftware.org not found: 3(NXDOMAIN)"},
                    {"message": "TIMEOUT"},
                ],
            },
            {
                "command": "dig {HOST}",
                "name": "network.dig",
                "successes": [{"message": "192.168.0.1"}],
            },
            {
                "command": "ping -c 1 {HOST}",
                "name": "network.ping",
                "failures": [
                    {
                        "message": "Name or service not known",
                        "exception": "InfrastructureError",
                        "error": "network setup failed",
                    }
                ],
            },
        ],
    }
    test_connection = Connection(conn_data)
    substitutions = {"{HOST}": "lavasoftware.org"}
    with pytest.raises(InfrastructureError) as exc:
        action.run_script(test_connection, script, substitutions)
    assert exc.match(  # nosec - assert is part of the test process.
        "network setup failed"
    )
    assert action.logger.data == []  # nosec - assert is part of the test process.
    assert test_connection.data == []  # nosec - assert is part of the test process.



# -*- coding: utf-8 -*-
from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("lava_scheduler_app", "0005_auto_devicedictionarytable_pipelinestore")
    ]

    operations = [
        migrations.CreateModel(
            name="ActionData",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("action_name", models.CharField(max_length=100)),
                ("action_level", models.CharField(max_length=32)),
                ("action_summary", models.CharField(max_length=100)),
                ("action_description", models.CharField(max_length=200)),
                ("yaml_line", models.PositiveIntegerField(null=True, blank=True)),
                (
                    "description_line",
                    models.PositiveIntegerField(null=True, blank=True),
                ),
                ("log_section", models.CharField(max_length=50, null=True, blank=True)),
                (
                    "duration",
                    models.DecimalField(
                        null=True, max_digits=8, decimal_places=2, blank=True
                    ),
                ),
                ("timeout", models.PositiveIntegerField(null=True, blank=True)),
                ("count", models.PositiveIntegerField(null=True, blank=True)),
                ("max_retries", models.PositiveIntegerField(null=True, blank=True)),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="MetaType",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("name", models.CharField(max_length=32)),
                (
                    "metatype",
                    models.PositiveIntegerField(
                        help_text="metadata action type",
                        verbose_name="Type",
                        choices=[
                            (0, "deploy"),
                            (1, "boot"),
                            (2, "test"),
                            (3, "diagnostic"),
                            (4, "finalize"),
                            (5, "unknown type"),
                        ],
                    ),
                ),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="TestCase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "name",
                    models.TextField(
                        help_text="Maximum length: 100 characters",
                        verbose_name="Name",
                        blank=True,
                    ),
                ),
                (
                    "units",
                    models.TextField(
                        help_text="Units in which measurement value should be\n                     interpreted, for example <q>ms</q>, <q>MB/s</q> etc.\n                     There is no semantic meaning inferred from the value of\n                     this field, free form text is allowed. <br/>Maximum length: 100 characters",
                        verbose_name="Units",
                        blank=True,
                    ),
                ),
                (
                    "result",
                    models.PositiveSmallIntegerField(
                        help_text="Result classification to pass/fail group",
                        verbose_name="Result",
                        choices=[
                            (0, "Test passed"),
                            (1, "Test failed"),
                            (2, "Test skipped"),
                            (3, "Unknown outcome"),
                        ],
                    ),
                ),
                (
                    "measurement",
                    models.CharField(
                        help_text="Arbitrary value that was measured as a part of this test.",
                        max_length=512,
                        null=True,
                        verbose_name="Measurement",
                        blank=True,
                    ),
                ),
                (
                    "metadata",
                    models.CharField(
                        help_text="Metadata collected by the pipeline action, stored as YAML.",
                        max_length=1024,
                        null=True,
                        verbose_name="Action meta data as a YAML string",
                        blank=True,
                    ),
                ),
                ("logged", models.DateTimeField(auto_now=True)),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="TestData",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "testjob",
                    models.ForeignKey(
                        related_name="test_data",
                        to="lava_scheduler_app.TestJob",
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="TestSet",
            fields=[
                ("id", models.AutoField(serialize=False, primary_key=True)),
                (
                    "name",
                    models.CharField(
                        default=None,
                        max_length=200,
                        null=True,
                        verbose_name="Suite name",
                        blank=True,
                    ),
                ),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="TestSuite",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        default=None,
                        max_length=200,
                        null=True,
                        verbose_name="Suite name",
                        blank=True,
                    ),
                ),
                (
                    "job",
                    models.ForeignKey(
                        related_name="test_suites",
                        to="lava_scheduler_app.TestJob",
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name="testset",
            name="suite",
            field=models.ForeignKey(
                related_name="test_sets",
                to="lava_results_app.TestSuite",
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="testcase",
            name="suite",
            field=models.ForeignKey(
                related_name="test_cases",
                to="lava_results_app.TestSuite",
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="testcase",
            name="test_set",
            field=models.ForeignKey(
                related_name="test_cases",
                default=None,
                blank=True,
                to="lava_results_app.TestSet",
                null=True,
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="actiondata",
            name="meta_type",
            field=models.ForeignKey(
                related_name="actionlevels",
                to="lava_results_app.MetaType",
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="actiondata",
            name="testcase",
            field=models.ForeignKey(
                related_name="actionlevels",
                blank=True,
                to="lava_results_app.TestCase",
                null=True,
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="actiondata",
            name="testdata",
            field=models.ForeignKey(
                related_name="actionlevels",
                blank=True,
                to="lava_results_app.TestData",
                null=True,
                on_delete=models.CASCADE,
            ),
            preserve_default=True,
        ),
    ]



# Copyright (C) 2014 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#         Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from lava_dispatcher.tests.test_basic import Factory, StdoutTestCase
from lava_dispatcher.actions.boot import AutoLoginAction


def find_autologin(job):
    for action in job.pipeline.actions:
        if action.internal_pipeline:
            for action in action.internal_pipeline.actions:
                if isinstance(action, AutoLoginAction):
                    return True
    return False


class TestMonitorPipeline(StdoutTestCase):
    def test_autologin_normal_kvm(self):
        factory = Factory()
        job = factory.create_kvm_job("sample_jobs/kvm.yaml")
        job.validate()
        self.assertTrue(find_autologin(job))

    def test_qemu_monitor_no_prompts(self):
        factory = Factory()
        job = factory.create_kvm_job("sample_jobs/qemu-monitor.yaml")
        job.validate()
        self.assertIsNotNone(job)
        self.assertIsNotNone(job.pipeline)
        self.assertIsNotNone(job.pipeline.actions)
        for action in job.pipeline.actions:
            action.validate()
            self.assertTrue(action.valid)
        self.assertFalse(find_autologin(job))

    def test_qemu_monitor_notest_noprompts(self):
        factory = Factory()
        job = factory.create_kvm_job("sample_jobs/kvm-notest-noprompts.yaml")
        job.validate()
        self.assertIsNotNone(job)
        self.assertIsNotNone(job.pipeline)
        self.assertIsNotNone(job.pipeline.actions)
        for action in job.pipeline.actions:
            action.validate()
            self.assertTrue(action.valid)
        self.assertFalse(find_autologin(job))

    def test_qemu_monitor_zephyr_job(self):
        factory = Factory()
        job = factory.create_kvm_job("sample_jobs/zephyr-qemu-test-task.yaml")
        job.validate()
        self.assertIsNotNone(job)
        self.assertIsNotNone(job.pipeline)
        self.assertIsNotNone(job.pipeline.actions)
        for action in job.pipeline.actions:
            action.validate()
            self.assertTrue(action.valid)
        self.assertFalse(find_autologin(job))

    def test_qemu_notest(self):
        factory = Factory()
        job = factory.create_kvm_job("sample_jobs/kvm-notest.yaml")
        job.validate()
        self.assertIsNotNone(job)
        self.assertIsNotNone(job.pipeline)
        self.assertIsNotNone(job.pipeline.actions)
        for action in job.pipeline.actions:
            action.validate()
            self.assertTrue(action.valid)
        self.assertTrue(find_autologin(job))



# Copyright (C) 2014 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.


from lava_dispatcher.action import Pipeline, Action
from lava_dispatcher.logical import Boot
from lava_dispatcher.actions.boot import BootAction
from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
from lava_dispatcher.shell import ExpectShellSession
from lava_dispatcher.actions.boot import AutoLoginAction


class BootKExec(Boot):
    """
    Expects a shell session, checks for kexec executable and
    prepares the arguments to run kexec,
    """

    compatibility = 1

    def __init__(self, parent, parameters):
        super().__init__(parent)
        self.action = BootKexecAction()
        self.action.section = self.action_type
        self.action.job = self.job
        parent.add_action(self.action, parameters)

    @classmethod
    def accepts(cls, device, parameters):
        if "method" in parameters:
            if parameters["method"] == "kexec":
                return True, "accepted"
        return False, '"method" was not in parameters, or "method" was not "kexec"'


class BootKexecAction(BootAction):
    """
    Provide for auto_login parameters in this boot stanza and re-establish the connection after boot
    """

    name = "kexec-boot"
    summary = "kexec a new kernel"
    description = "replace current kernel using kexec"

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        self.internal_pipeline.add_action(KexecAction())
        # Add AutoLoginAction unconditionally as this action does nothing if
        # the configuration does not contain 'auto_login'
        self.internal_pipeline.add_action(AutoLoginAction())
        self.internal_pipeline.add_action(ExpectShellSession())
        self.internal_pipeline.add_action(ExportDeviceEnvironment())


class KexecAction(Action):
    """
    The files need to have been downloaded by a previous test action.
    This action calls kexec to load the kernel ,execute it and then
    attempts to reestablish the shell connection after boot.
    """

    name = "call-kexec"
    summary = "attempt to kexec new kernel"
    description = "call kexec with specified arguments"

    def __init__(self):
        super().__init__()
        self.command = ""
        self.load_command = ""

    def validate(self):
        super().validate()
        self.command = self.parameters.get("command", "/sbin/kexec")
        self.load_command = self.command[:]  # local copy for idempotency
        self.command += " -e"
        if "kernel" in self.parameters:
            self.load_command += " --load %s" % self.parameters["kernel"]
        if "dtb" in self.parameters:
            self.load_command += " --dtb %s" % self.parameters["dtb"]
        if "initrd" in self.parameters:
            self.load_command += " --initrd %s" % self.parameters["initrd"]
        if "options" in self.parameters:
            for option in self.parameters["options"]:
                self.load_command += " %s" % option
        if self.load_command == "/sbin/kexec":
            self.errors = "Default kexec handler needs at least a kernel to pass to the --load command"

    def run(self, connection, max_end_time):
        """
        If kexec fails, there is no real chance at diagnostics because the device will be hung.
        Get the output prior to the call, in case this helps after the job fails.
        """
        connection = super().run(connection, max_end_time)
        if "kernel-config" in self.parameters:
            cmd = "zgrep -i kexec %s |grep -v '^#'" % self.parameters["kernel-config"]
            self.logger.debug("Checking for kexec: %s", cmd)
            connection.sendline(cmd)
        connection.sendline(self.load_command)
        self.wait(connection)
        connection.prompt = self.parameters["boot_message"]
        connection.sendline(self.command)
        return connection



# coding: utf-8
#
# Copyright (C) 2018 Linaro Limited
#
# Author: Rémi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from voluptuous import All, Length, Any, Match, Optional, Required

from lava_common.schemas import test


def schema(live=False):
    base = {
        Required("interactive"): [
            {
                Required("name"): str,
                Required("prompts"): [All(str, Length(min=1))],
                Required("script"): [
                    {
                        Required("command"): Any(str, None),
                        Optional("name"): All(str, Length(min=1)),
                        Optional("successes"): [
                            {Required("message"): All(str, Length(min=1))}
                        ],
                        Optional("failures"): [
                            {
                                Required("message"): All(str, Length(min=1)),
                                Optional("exception"): Any(
                                    "InfrastructureError", "JobError", "TestError"
                                ),
                                Optional("error"): All(str, Length(min=1)),
                            }
                        ],
                    }
                ],
            }
        ]
    }
    return {**test.schema(live), **base}
